//
// Ammo.cpp
//

#include "Ammo.h"
#include "Engine/GraphicsManager.h"
#include <sstream>
#include "Engine/WorldManager.h"
#include "Engine/LogManager.h"

Ammo::Ammo() {
	ammo = -1;
	gun_type = "standard";
	setSolid(false);
	altitude = 2;
	WorldManager &wm = WorldManager::getInstance();
	Position init_pos(22,wm.getView().getVertical() - 1);
	setPos(init_pos);
	wm.registerInterest(this, "EVENT_AMMO");
	wm.registerInterest(this, "EVENT_GUN_CHANGE");
}

Ammo::Ammo(int init_ammo, string init_gun_type) {
	ammo = init_ammo;
	gun_type = init_gun_type;
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_AMMO");
	wm.registerInterest(this, "EVENT_GUN_CHANGE");
};

Ammo::~Ammo() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_AMMO");
	wm.unregisterInterest(this, "EVENT_GUN_CHANGE");
};

int Ammo::getAmmo() {
	return ammo;
};

void Ammo::setAmmo(int new_ammo) {
	ammo = new_ammo;
};

string Ammo::getGunType() {
	return gun_type;
};

void Ammo::setGunType(string new_gun_type) {
	gun_type = new_gun_type;
};

void Ammo::draw() {
	int color = COLOR_YELLOW;
	string s_ten;
	string s_one;
	if(ammo >= 0) {
		int tens = ammo / 10;
		stringstream out_ten;
		out_ten << tens;
		s_ten = out_ten.str();
		
		int ones = ammo % 10;
		stringstream out_one;
		out_one << ones;
		s_one = out_one.str();
	};
	
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	graph_man.drawCh(pos, 'A',color);
	int new_x = pos.getX() + 1;
	Position new_pos(new_x, pos.getY());
	graph_man.drawCh(new_pos, 'M',color);
	new_x++;
	new_pos.setX(new_x);
	graph_man.drawCh(new_pos, 'M',color);
	new_x++;
	new_pos.setX(new_x);
	graph_man.drawCh(new_pos, 'O',color);
	new_x++;
	new_pos.setX(new_x);
	graph_man.drawCh(new_pos, ':',color);
	new_x++;
	new_pos.setX(new_x);
	if(ammo >= 0) {
		graph_man.drawCh(new_pos, s_ten.c_str()[0],color);
	}
	else {
		graph_man.drawCh(new_pos, '-',color);
	};
	new_x++;
	new_pos.setX(new_x);
	if(ammo >= 0) {
		graph_man.drawCh(new_pos, s_one.c_str()[0],color);
	}
	else {
		graph_man.drawCh(new_pos, '-',color);
	};
};

int Ammo::eventHandler(Event *e) {
	if(e->getType() == "EVENT_AMMO") {
		EventAmmo *e_ammo = static_cast <EventAmmo *> (e);
		changeAmmo(e_ammo);
	}
	else if(e->getType() == "EVENT_GUN_CHANGE") {
		EventGunChange *e_change = static_cast <EventGunChange *> (e);
		ammo = e_change->getAmmo();
		gun_type = e_change->getGunType();
	}
};

void Ammo::changeAmmo(EventAmmo *e) {
	LogManager &lm = LogManager::getInstance();
	string e_gun_type = e->getGunType();
	lm.writeLog("Gun type is %s", e_gun_type.c_str());
	if(e_gun_type == gun_type) {
		ammo = e->getAmmo();
	};
};


