//
// BigEnemy.cpp
//

#include "BigEnemy.h"
#include "EnemyShot.h"
#include "Bullet.h"
#include "EnemyFactory.h"
#include "EventBoss.h"
#include <stdlib.h>
#include <time.h>

BigEnemy::BigEnemy(Position init_pos) {
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	pos = init_pos;
	altitude = 0;
	dir = 0;
	ResourceManager &rm = ResourceManager::getInstance();
	p_sprite = rm.getSprite("bigEnemy");
	sprite_index = 0;
	sprite_slowdown = 60;
	sprite_slowdown_count = 0;
	box.setCorner(pos);
	box.setHorizontal(p_sprite->getWidth());
	box.setVertical(p_sprite->getHeight());
	shot_counter = 45;
	move_counter = 0;
	health = 20;
	type = "BigEnemy";
	setSolid(true);
	srand(time(0));
};

BigEnemy::~BigEnemy() {	
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
};

int BigEnemy::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	GameManager &gm = GameManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	WorldManager &wm = WorldManager::getInstance();

	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		lm.writeLog("Enemy recieved a collision event");
		if(coll_e->getObj()->getType() == "bullet") {
			Bullet *bullet = static_cast <Bullet *> (coll_e->getObj());
			health = health - bullet->getDamage();
			EventBoss *e_boss = new EventBoss(health);
			wm.onEvent(e_boss);
		}
		if(health <= 0){
			wm.markForDelete(this);
			ef.wipeScreen();
			ef.setLevel(2);
			ef.startLevel();
		}
	}
	if(e->getType() == "EVENT_STEP") {
		move_counter--;
		shot_counter--;
		if(move_counter<=0){
			move();
			move_counter = 50;
		}
		if(shot_counter<=0){
			int x = pos.getX();
			int y = pos.getY();
				Position pos1(x+2,y-1);
				Position pos2(x+3,y-1);
				Position pos3(x+4,y-1);
				Position pos4(x+5,y-1);
				Position pos5(x+6,y-1);
				Position pos6(x+7,y-1);
				Position pos7(x+10,y+2);
				Position pos8(x+10,y+3);
				Position pos9(x+10,y+4);
				Position pos10(x+10,y+5);
				Position pos11(x+10,y+6);
				Position pos12(x+10,y+7);
				Position pos13(x+10,y+8);
				Position pos14(x+10,y+9);
				Position pos15(x+10,y+10);
				Position pos16(x+2,y+13);
				Position pos17(x+3,y+13);
				Position pos18(x+4,y+13);
				Position pos19(x+5,y+13);
				Position pos20(x+6,y+13);
				Position pos21(x+7,y+13);
				Position pos22(x-1,y+2);
				Position pos23(x-1,y+3);
				Position pos24(x-1,y+4);
				Position pos25(x-1,y+5);
				Position pos26(x-1,y+6);
				Position pos27(x-1,y+7);
				Position pos28(x-1,y+8);
				Position pos29(x-1,y+9);
				Position pos30(x-1,y+10);
			switch(dir){
			case 1:
				new EnemyShot(1,5,pos1);
				new EnemyShot(1,5,pos2);
				new EnemyShot(1,5,pos3);
				new EnemyShot(1,5,pos4);
				new EnemyShot(1,5,pos5);
				new EnemyShot(1,5,pos6);
				break;
			case 2:
				new EnemyShot(1,5,pos1);
				new EnemyShot(1,5,pos2);
				new EnemyShot(1,5,pos3);
				new EnemyShot(1,5,pos4);
				new EnemyShot(1,5,pos5);
				new EnemyShot(1,5,pos6);
				new EnemyShot(3,5,pos7);
				//new EnemyShot(3,5,pos8);
				new EnemyShot(3,5,pos9);
				//new EnemyShot(3,5,pos10);
				new EnemyShot(3,5,pos11);
				//new EnemyShot(3,5,pos12);
				new EnemyShot(3,5,pos13);
				//new EnemyShot(3,5,pos14);
				new EnemyShot(3,5,pos15);
				break;
			case 3:
				new EnemyShot(3,5,pos7);
				new EnemyShot(3,5,pos8);
				new EnemyShot(3,5,pos9);
				new EnemyShot(3,5,pos10);
				new EnemyShot(3,5,pos11);
				new EnemyShot(3,5,pos12);
				new EnemyShot(3,5,pos13);
				new EnemyShot(3,5,pos14);
				new EnemyShot(3,5,pos15);
				break;
			case 4:
				new EnemyShot(5,5,pos16);
				new EnemyShot(5,5,pos17);
				new EnemyShot(5,5,pos18);
				new EnemyShot(5,5,pos19);
				new EnemyShot(5,5,pos20);
				new EnemyShot(5,5,pos21);
				new EnemyShot(3,5,pos7);
				new EnemyShot(3,5,pos8);
				new EnemyShot(3,5,pos9);
				new EnemyShot(3,5,pos10);
				new EnemyShot(3,5,pos11);
				new EnemyShot(3,5,pos12);
				new EnemyShot(3,5,pos13);
				new EnemyShot(3,5,pos14);
				new EnemyShot(3,5,pos15);
				break;
			case 5:
				new EnemyShot(5,5,pos16);
				new EnemyShot(5,5,pos17);
				new EnemyShot(5,5,pos18);
				new EnemyShot(5,5,pos19);
				new EnemyShot(5,5,pos20);
				new EnemyShot(5,5,pos21);
				break;
			case 6:
				new EnemyShot(5,5,pos16);
				new EnemyShot(5,5,pos17);
				new EnemyShot(5,5,pos18);
				new EnemyShot(5,5,pos19);
				new EnemyShot(5,5,pos20);
				new EnemyShot(5,5,pos21);
				new EnemyShot(7,5,pos22);
				new EnemyShot(7,5,pos23);
				new EnemyShot(7,5,pos24);
				new EnemyShot(7,5,pos25);
				new EnemyShot(7,5,pos26);
				new EnemyShot(7,5,pos27);
				new EnemyShot(7,5,pos28);
				new EnemyShot(7,5,pos29);
				new EnemyShot(7,5,pos30);
				break;
			case 7:
				new EnemyShot(7,5,pos22);
				//new EnemyShot(7,5,pos23);
				new EnemyShot(7,5,pos24);
				//new EnemyShot(7,5,pos25);
				new EnemyShot(7,5,pos26);
				//new EnemyShot(7,5,pos27);
				new EnemyShot(7,5,pos28);
				//new EnemyShot(7,5,pos29);
				new EnemyShot(7,5,pos30);
				break;
			case 8:
				new EnemyShot(1,5,pos1);
				new EnemyShot(1,5,pos2);
				new EnemyShot(1,5,pos3);
				new EnemyShot(1,5,pos4);
				new EnemyShot(1,5,pos5);
				new EnemyShot(1,5,pos6);
				new EnemyShot(7,5,pos22);
				//new EnemyShot(7,5,pos23);
				new EnemyShot(7,5,pos24);
				//new EnemyShot(7,5,pos25);
				new EnemyShot(7,5,pos26);
				//new EnemyShot(7,5,pos27);
				new EnemyShot(7,5,pos28);
				//new EnemyShot(7,5,pos29);
				new EnemyShot(7,5,pos30);
				break;
			}
			shot_counter = rand()%25+100;
		}
	}
	return 0;
};

void BigEnemy::move() {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	int p_x = pos.getX();
	int p_y = pos.getY();
    int hero_pos_x = ef.getHero().getPos().getX();
	int hero_pos_y = ef.getHero().getPos().getY();

    if(p_x < hero_pos_x && p_x + 10 > hero_pos_x && p_y > hero_pos_y)
		dir = 1;
	if(p_x + 10 < hero_pos_x && p_y > hero_pos_y + 2)
		dir = 2;
	if(p_x + 10 < hero_pos_x && p_y < hero_pos_y && p_y + 13 > hero_pos_y +2)
		dir = 3;
	if(p_x + 10 < hero_pos_x && p_y + 13 < hero_pos_y)
		dir = 4;
	if(p_x < hero_pos_x && p_x + 10 > hero_pos_x && p_y + 13 < hero_pos_y)
		dir = 5;
	if(p_x > hero_pos_x + 2 && p_y + 13 < hero_pos_y)
		dir = 6;
	if(p_x > hero_pos_x + 2 && p_y < hero_pos_y && p_y + 13 > hero_pos_y + 2)
		dir = 7;
    if(p_x > hero_pos_x + 2 && p_y > hero_pos_y + 2)
		dir = 8;

	switch(dir){
	case 1:
		p_y--;
		break;
	case 2:
		p_x+=2;
		p_y--;
		break;
	case 3:
		p_x+=2;
		break;
	case 4:
		p_x+=2;
		p_y++;
		break;
	case 5:
		p_y++;
		break;
	case 6:
		p_x-=2;
		p_y++;
		break;
	case 7:
		p_x-=2;
		break;
	case 8:
		p_x-=2;
		p_y--;
		break;
	}

	Position new_pos(p_x,p_y);
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == new_pos.getY()) {
		box.setCorner(new_pos);
	};
};
