//
// Bullet.cpp
//

#include "Bullet.h"

Bullet::Bullet() {
	setType("bullet");
	altitude = 0;
	setSolid(true);
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	box.setCorner(pos);
	box.setHorizontal(1);
	box.setVertical(1);
	init_counter = 2;
	counter = init_counter;
};

Bullet::Bullet(int new_direction, Position new_pos) {
	setType("bullet");
	altitude = 0;
	setSolid(true);
	direction = new_direction;
	pos.setX(new_pos.getX());
	pos.setY(new_pos.getY());
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	box.setCorner(pos);
	box.setHorizontal(1);
	box.setVertical(1);
	init_counter = 2;
	counter = init_counter;
};

Bullet::~Bullet() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
};

int Bullet::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	if(e->getType() == "EVENT_STEP") {
		counter--;
		if(counter <= 0) {
			counter = init_counter;
			move();
		};
	}
	if(e->getType() == "EVENT_COLLISION") {
		WorldManager &wm = WorldManager::getInstance();
		wm.markForDelete(this);
	}
	if(e->getType() == "EVENT_OFFSCREEN") {
		WorldManager &wm = WorldManager::getInstance();
		wm.markForDelete(this);
	}
};

int Bullet::getDirection() {
	return direction;
};

void Bullet::draw() {
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	Position new_pos = graph_man.worldToScreen(pos);
	graph_man.drawCh(new_pos, '7');
};

void Bullet::move() {
	int x = pos.getX();
	int y = pos.getY();
	if(direction == UP) {
		y--;
	}
	else if(direction == DOWN) {
		y++;
	}
	else if(direction == LEFT) {
		x = x-2;
	}
	else if(direction == RIGHT) {
		x = x+2;
	}
	WorldManager &wm = WorldManager::getInstance();
	Position new_pos(x,y);
	if(distance > 0) {
		distance--;
	};
	if(x < 0 || y < 0 || x > wm.getBoundary().getHorizontal() || y > wm.getBoundary().getVertical() || distance == 0) {
		wm.markForDelete(this);
		return;		
	}
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == pos.getY()) {
		box.setCorner(new_pos);
	}
};

int Bullet::getDamage() {
	return damage;
};
