///
/// EnemyFactory.cpp
///

#include "EnemyFactory.h"
#include "EventBoss.h"
#include "BossHealth.h"
#include "BigEnemy.h"
#include "SecondBoss.h"
#include "EventLife.h"
#include "Engine/GameObjectListIterator.h"
#include "Engine/GraphicsManager.h"
#include "Engine/LogManager.h"
#include "Hero.h"
#include "Timer.h"
#include "Lives.h"
#include "Ammo.h"
#include "AmmoDrop.h"
#include "LifeDrop.h"
#include "GunLabel.h"
#include "Title.h"
#include "Level2.h"
#include "Level3.h"
#include "Instructions.h"
#include "Instructions2.h"
#include "Instructions3.h"
#include <stdlib.h>
#include <time.h>

EnemyFactory::EnemyFactory() {
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this,"EVENT_LIFE");
	wm.registerInterest(this,"EVENT_STEP");
	start_up = 300;
	level = 1;
	curr_MO = 0;
	curr_SO = 0;
	curr_enem1 = 0;
	curr_enem2 = 0;
	max_MO = 0;
	max_SO = 0;
	max_enem1 = 0;
	max_enem2 = 0;
	altitude = -2;
	Position p(0,0);
	pos = p;
	life = 3;
	srand(time(0));
	bossFight = false;
	spawn_delay = 30;
	Hero hero();
};

EnemyFactory::EnemyFactory(EnemyFactory const&) {

};

void EnemyFactory::operator=(EnemyFactory const&) {

};

EnemyFactory::~EnemyFactory() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this,"EVENT_LIFE");
	wm.unregisterInterest(this,"EVENT_STEP");
};

EnemyFactory &EnemyFactory::getInstance() {
  static EnemyFactory instance;
  return instance;
};

void EnemyFactory::startLevel(){
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	bossFight = false;
	Hero hero();
	new Timer;
	switch(level){
	case 1:
		new Title;
		new Instructions;
		while(curr_SO<max_SO){
			Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
			Box box(p,1,1);
			if(checkPosition(box) && p.getY() != 30){
				new StaticObstacle(p);
			}
		}
		/*while(curr_enem1<max_enem1){
			Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
			Box box(p,2,2);
			if(checkPosition(box) && offScreen(box)){
				new FirstEnemy(p);
			}
		}
		while(curr_enem2<max_enem2){
			Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
			Box box(p,3,3);
			if(checkPosition(box) && offScreen(box)){
				new SecondEnemy(p);
			}
		}*/
		break;
	case 2:
		start_up = 300;
		new Instructions2;
		new Level2;
		lm.writeLog("building level 2");
		while(curr_SO<max_SO){
			Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
			Box box(p,1,1);
			if(checkPosition(box) && p.getY() != 30){
				new StaticObstacle(p);
			}
		}
		/*while(curr_enem1<max_enem1){
			Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
			Box box(p,2,2);
			if(checkPosition(box) && offScreen(box)){
				new FirstEnemy(p);
			}
		}
		while(curr_enem2<max_enem2){
			Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
			Box box(p,3,3);
			if(checkPosition(box) && offScreen(box)){
				new SecondEnemy(p);
			}
		}
		while(curr_enemS<max_enemS){
			Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
			Box box(p,2,3);
			if(checkPosition(box) && offScreen(box)){
				new SmartEnemy(p);
			}
		}*/
		break;
	case 3:
		start_up = 300;
		new Instructions3;
		new Level3;
		lm.writeLog("building level 3");
		while(curr_SO<max_SO){
			Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
			Box box(p,1,1);
			if(checkPosition(box) && p.getY() != 30){
				new StaticObstacle(p);
			}
		}
		break;
	}
};

void EnemyFactory::bossSpawn(){
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	EventBoss *e_boss = new EventBoss(15);
	new BossHealth();
	wm.onEvent(e_boss);

	GameObjectListIterator iter = enemies.createIterator();
	iter.first();
	while(!iter.isDone()){
		GameObject *temp = iter.currentObj();
		if(temp->getType() != "SO"){
			wm.markForDelete(temp);
		}
		iter.next();
	};

	switch(level){
	case 1:{
		Position p(15,15);
		Box box(p,13,10);
		while(!checkPosition(box)){
			int y = p.getY();
			y++;
			p.setY(y);
			box.setCorner(p);
		}
		boss = new BigEnemy(p);
		bossFight = true;
		EventBoss *e_boss = new EventBoss(20);
		new BossHealth();
		wm.onEvent(e_boss);
		}
		break;
	case 2:{
		Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
		Box box(p,7,7);
		while(!checkPosition(box)){
			p.setX(rand()%wm.getBoundary().getHorizontal());
			p.setY(rand()%wm.getBoundary().getVertical());
			box.setCorner(p);
		}
		boss = new SecondBoss(p);
		bossFight = true;
		EventBoss *e_boss = new EventBoss(20);
		new BossHealth();
		wm.onEvent(e_boss);
		}
		break;
	case 3:{
		int counter=20;
		Position p(15,15);
		Box box(p,13,10);
		while(!checkPosition(box)){
			int y = p.getY();
			y++;
			p.setY(y);
			box.setCorner(p);
		}
		boss = new BigEnemy(p);
		bossFight = true;
		EventBoss *e_boss = new EventBoss(20);
		new BossHealth();
		wm.onEvent(e_boss);
		while(counter>=0){
			Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
			Box box(p,2,3);
			if(checkPosition(box) && offScreen(box)){
				new SmartEnemy(p);
				counter--;
			}
		}
		   }
		break;
	}


};

void EnemyFactory::addEnemy(GameObject *enemy){
	enemies.insert(enemy);
	int type = 0;
	if (enemy->getType() == "FirstEnemy")
		type = 1;
	if (enemy->getType() == "SecondEnemy")
		type = 2;
	if (enemy->getType() == "MO")
		type = 3;
	if (enemy->getType() == "SO")
		type = 4;
	if (enemy->getType() == "SmartEnemy")
		type = 5;
	switch(type){
	case 1:
		curr_enem1++;
		break;
	case 2:
		curr_enem2++;
		break;
	case 3:
		curr_MO++;
		break;
	case 4:
		curr_SO++;
		break;
	case 5:
		curr_enemS++;
		break;
	}
};

void EnemyFactory::removeEnemy(GameObject *enemy){
	enemies.remove(enemy);
	int type = 0;
	if (enemy->getType() == "FirstEnemy")
		type = 1;
	if (enemy->getType() == "SecondEnemy")
		type = 2;
	if (enemy->getType() == "MO")
		type = 3;
	if (enemy->getType() == "SO")
		type = 4;
	if (enemy->getType() == "SmartEnemy")
		type = 5;
	switch(type){
	case 1:
		curr_enem1--;
		break;
	case 2:
		curr_enem2--;
		break;
	case 3:
		curr_MO--;
		break;
	case 4:
		curr_SO--;
		break;
	case 5:
		curr_enemS--;
		break;
	}
};

int EnemyFactory::eventHandler(Event *e){
	WorldManager &wm = WorldManager::getInstance();
	GraphicsManager &grm = GraphicsManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	if(e->getType() == "EVENT_LIFE" && !bossFight) {
		EventLife *life_e = static_cast <EventLife *> (e);
		if(life_e->getLives() < life) {
			life = life_e->getLives();
			GameObjectListIterator iter = enemies.createIterator();
			iter.first();
			while(!iter.isDone()){
				GameObject *temp = iter.currentObj();
				if(!offScreen(temp->getBox()) && temp->getType() != "SO"){
					wm.markForDelete(temp);
				}
				iter.next();
			}
		}
	}
	if(e->getType() == "EVENT_STEP"){
		if(start_up > 0) {
			start_up--;
		}
		else {
			spawn_delay--;
			if(spawn_delay<=0 && !bossFight){
				switch(level){
				case 1:{
					int max_enemies = 30;
					int new_amount = 10;
					int counter = 0;
					while(curr_enem1 + curr_enem2 + curr_enemS <max_enemies && counter <= new_amount){
						int spawn_chance = rand()%10+1;
						Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
						Box box(p,3,5);
						if(checkPosition(box) && offScreen(box)){
							if(spawn_chance >= 5){
								new FirstEnemy(p);
							}
							if(spawn_chance<5){
								new SecondEnemy(p);
							}
							counter++;
						}
					}
					while(curr_MO<max_MO){
						Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
						Box box(p,1,1);
						if(checkPosition(box) && offScreen(box)){
							new MovingObstacle(p,rand()%8+1);
						}
					}
					while(curr_SO<max_SO){
						Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
						Box box(p,1,1);
						if(checkPosition(box) && offScreen(box)){
							new StaticObstacle(p);
						}
					}
						}
					break;
				case 2:{
					int max_enemies = 45;
					int new_amount = 12;
					int counter = 0;
					while(curr_enem1 + curr_enem2 + curr_enemS <max_enemies && counter <= new_amount){
						int spawn_chance = rand()%10+1;
						Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
						Box box(p,3,5);
						if(checkPosition(box) && offScreen(box)){
							if(spawn_chance >= 6){
								new FirstEnemy(p);
							}
							if(spawn_chance==5){
								new SecondEnemy(p);
							}
							if(spawn_chance<=4){
								new SmartEnemy(p);
							}
							counter++;
						}
					}
					while(curr_MO<max_MO){
						Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
						Box box(p,1,1);
						if(checkPosition(box) && offScreen(box)){
							new MovingObstacle(p,rand()%8+1);
						}
					}
					while(curr_SO<max_SO){
						Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
						Box box(p,1,1);
						if(checkPosition(box) && offScreen(box)){
							new StaticObstacle(p);
						}
					}
						}
					break;
				case 3:{
					int max_enemies = 50;
					int new_amount = 15;
					int counter = 0;
					while(curr_enem1 + curr_enem2 + curr_enemS <max_enemies && counter <= new_amount){
						int spawn_chance = rand()%10+1;
						Position p(rand()%wm.getBoundary().getHorizontal(),rand()%wm.getBoundary().getVertical());
						Box box(p,3,5);
						if(checkPosition(box) && offScreen(box)){
							if(spawn_chance >= 8){
								new FirstEnemy(p);
							}
							if(spawn_chance==7){
								new SecondEnemy(p);
							}
							if(spawn_chance<=6){
								new SmartEnemy(p);
							}
							counter++;
						}
					}
					while(curr_MO<max_MO){
						Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
						Box box(p,1,1);
						if(checkPosition(box) && offScreen(box)){
							new MovingObstacle(p,rand()%8+1);
						}
					}
					while(curr_SO<max_SO){
						Position p((rand()%wm.getBoundary().getHorizontal()/2 * 2)+1,rand()%wm.getBoundary().getVertical());
						Box box(p,1,1);
						if(checkPosition(box) && offScreen(box)){
							new StaticObstacle(p);
						}
					}
						}
					break;
				}
				spawn_delay = 60;
			}
		}
	}

};

bool EnemyFactory::checkPosition(Box enem_box){
	WorldManager &wm = WorldManager::getInstance();
	GameObjectListIterator iter = enemies.createIterator();
	iter.first();
	while(!iter.isDone()){
		GameObject *temp = iter.currentObj();
		if(wm.boxesIntersect(temp->getBox(),enem_box)){
			return false;
		}
		iter.next();
	};
	if(wm.boxesIntersect(hero.getBox(),enem_box)){
		return false;
	}
	return true;
};

bool EnemyFactory::offScreen(Box enem_box){
	WorldManager &wm = WorldManager::getInstance();
	GraphicsManager &grm = GraphicsManager::getInstance();
	Position p;
	p = grm.worldToScreen(enem_box.getCorner());
	int x = p.getX();
	int y = p.getY();
	if(x<0 || y<0 || x>wm.getView().getHorizontal() || y>wm.getView().getVertical()){
		return true;
	}
};

void EnemyFactory::draw(){

};

int EnemyFactory::getLevel(){
	return level;
};

void EnemyFactory::setLevel(int new_level){
	level = new_level;
	switch(level){
	case 1:
		max_MO = 15;
		max_SO = 50;
		max_enem1 = 10;
		max_enem2 = 15;
		max_enemS = 0;
		boss = NULL;
		break;
	case 2:
		max_MO = 25;
		max_SO = 15;
		max_enem1 = 15;
		max_enem2 = 10;
		max_enemS = 10;
		boss = NULL;
		break;
	case 3:
		max_MO = 25;
		max_SO = 20;
		max_enem1 = 10;
		max_enem2 = 13;
		max_enemS = 15;
		boss = NULL;
		break;
	}
};

Hero &EnemyFactory::getHero(){
	return hero;
};

void EnemyFactory::wipeScreen(){
	WorldManager &wm = WorldManager::getInstance();
	GameObjectListIterator iter = enemies.createIterator();
	iter.first();
	while(!iter.isDone()){
		GameObject *temp = iter.currentObj();
		wm.markForDelete(temp);
		iter.next();
	};
};

GameObject * EnemyFactory::getBoss(){
	return boss;
};

bool EnemyFactory::isBoss(){
	return bossFight;
};

void EnemyFactory::spawnAmmo(Position new_pos) {
	if(start_up == 0) {
		if(getLevel() == 1) {
			int num = rand()%100;
			if(num < 34) {
				int ammo = rand()%2 + 1;
				new AmmoDrop("second",ammo,new_pos);
			}
			else if(num < 39) {
				new LifeDrop(new_pos);
			}
		}
		else if(getLevel() == 2) {
			int num = rand()%100;
			if(num < 20) {
				int ammo = rand()%2 + 1;
				new AmmoDrop("second",ammo,new_pos);
			}
			else if(num < 40) {
				int ammo = rand()%2 + 1;
				new AmmoDrop("scatter",ammo,new_pos);
			}
			else if(num < 50) {
				int ammo = 1;
				new AmmoDrop("rail",ammo,new_pos);
			}
			else if(num < 55) {
				new LifeDrop(new_pos);
			}
		}
		else if(getLevel() >= 3) {
			int num = rand()%100;
			if(num < 18) {
				int ammo = rand()%2 + 1;
				new AmmoDrop("second",ammo,new_pos);
			}
			else if(num < 36) {
				int ammo = rand()%2 + 1;
				new AmmoDrop("scatter",ammo,new_pos);
			}
			else if(num < 48) {
				int ammo = 1;
				new AmmoDrop("rail",ammo,new_pos);
			}
			else if(num < 60) {
				int ammo = 1;
				new AmmoDrop("rocket",ammo,new_pos);
			}
			else if(num < 65) {
				new LifeDrop(new_pos);
			}
		};
	};
};

