//
// EnemyShot.cpp
//

#include "EnemyShot.h"

EnemyShot::EnemyShot(int init_direction,int init_counter,Position init_pos) {
	setType("EnemyShot");
	altitude = 0;
	setSolid(true);
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	box.setCorner(pos);
	box.setHorizontal(1);
	box.setVertical(1);
	direction = init_direction;
	counter = init_counter;
	count = init_counter;
	pos = init_pos;
	lifespan = 20;
};

EnemyShot::~EnemyShot() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
};

int EnemyShot::eventHandler(Event *e) {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	if(e->getType() == "EVENT_STEP") {	
		counter--;
		if(counter <= 0) {
			lifespan--;
			if(lifespan<=0){
				wm.markForDelete(this);
				return 0;
			}	
			counter = count;
			move();
		};
	}
	if(e->getType() == "EVENT_COLLISION") {
		lm.writeLog("EnemyShot recieved collision event");
		
		wm.markForDelete(this);
	}
	if(e->getType() == "EVENT_OUT") {
		GraphicsManager &gm = GraphicsManager::getInstance();
		Position new_pos = gm.worldToScreen(pos);
		lm.writeLog("Out event recieved at screen pos %d, %d; world pos %d, %d", new_pos.getX(), new_pos.getY(), pos.getX(), pos.getY());
		WorldManager &wm = WorldManager::getInstance();
		wm.markForDelete(this);
	}
};

void EnemyShot::draw() {
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	Position new_pos = graph_man.worldToScreen(pos);
	graph_man.drawCh(new_pos, 'x');
};

void EnemyShot::move() {
	int x = pos.getX();
	int y = pos.getY();

	switch(direction){
	case 1:
		y--;
		break;
	case 2:
		y--;
		x+=2;
		break;
	case 3:
		x+=2;
		break;
	case 4:
		x+=2;
		y++;
		break;
	case 5:
		y++;
		break;
	case 6:
		y++;
		x-=2;
		break;
	case 7:
		x-=2;
		break;
	case 8:
		y--;
		x-=2;
		break;
	}

	WorldManager &wm = WorldManager::getInstance();
	Position new_pos(x,y);
	if(x < 0 || y < 0 || x > wm.getBoundary().getHorizontal() || y > wm.getBoundary().getVertical()) {
		wm.markForDelete(this);
		return;		
	}
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == pos.getY()) {
		box.setCorner(new_pos);
	}
};
