//
//GameManager.cpp
//

#include "GameManager.h"
#include "LogManager.h"
#include "GraphicsManager.h"
#include "InputManager.h"
#include "Box.h"
#include <cstdlib>
#include <stdio.h>

GameManager::GameManager() {
  game_over = false;
};

GameManager::GameManager(GameManager const&) {

};

void GameManager::operator=(GameManager const&) {

};

GameManager::~GameManager() {

};

GameManager &GameManager::getInstance() {
  static GameManager instance;
  return instance;
};

int GameManager::startUp() {
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  GraphicsManager &graphics_manager = GraphicsManager::getInstance();
  InputManager &input_manager = InputManager::getInstance();
  log_manager.startUp(false,false);
  world_manager.startUp();
  graphics_manager.startUp();
  input_manager.startUp();
  Position world_corner(0,0);
  Box boundary(world_corner, graphics_manager.getHorizontal()-1,
    graphics_manager.getVertical()-1);
  world_manager.setBoundary(boundary);
  world_manager.setView(boundary);
  log_manager.writeLog("World view and boundary dimensions are: %i x %i, at %i,%i",
    world_manager.getBoundary().getHorizontal(), world_manager.getBoundary().getHorizontal(),
      world_manager.getView().getCorner().getX(), world_manager.getView().getCorner().getY());
  return 0;
};

int GameManager::startUp(bool append, bool flush) {
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  GraphicsManager &graphics_manager = GraphicsManager::getInstance();
  InputManager &input_manager = InputManager::getInstance();
  log_manager.startUp(append,flush);
  world_manager.startUp();
  graphics_manager.startUp();
  input_manager.startUp();
  Position world_corner(0,0);
  Box boundary(world_corner, graphics_manager.getHorizontal()-1,
    graphics_manager.getVertical()-1);
  world_manager.setBoundary(boundary);
  world_manager.setView(boundary);
  log_manager.writeLog("World view and boundary dimensions are: %i x %i, at %i,%i",
    world_manager.getBoundary().getHorizontal(), world_manager.getBoundary().getHorizontal(),
      world_manager.getView().getCorner().getX(), world_manager.getView().getCorner().getY());
  return 0;
};

int GameManager::startUp(bool append, bool flush, time_t seed) {
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  GraphicsManager &graphics_manager = GraphicsManager::getInstance();
  InputManager &input_manager = InputManager::getInstance();
  log_manager.startUp(append,flush);
  world_manager.startUp();
  graphics_manager.startUp();
  input_manager.startUp();
  Position world_corner(0,0);
  Box boundary(world_corner, graphics_manager.getHorizontal()-1,
    graphics_manager.getVertical()-1);
  world_manager.setBoundary(boundary);
  world_manager.setView(boundary);
  log_manager.writeLog("World view and boundary dimensions are: %i x %i, at %i,%i",
    world_manager.getBoundary().getHorizontal(), world_manager.getBoundary().getHorizontal(),
      world_manager.getView().getCorner().getX(), world_manager.getView().getCorner().getY());
  return 0;
};

void GameManager::run() {
  Clock *clock = new Clock();
  int counter = 1;

  LogManager &lm = LogManager::getInstance();
  WorldManager &wm = WorldManager::getInstance();
  GraphicsManager &graphics_manager = GraphicsManager::getInstance();
  InputManager &input_manager = InputManager::getInstance();

  // game loop
  while(!game_over) {
    input_manager.getInput();

    wm.update();
	
	wm.draw();
    
    graphics_manager.swapBuffers();

    float last_time = clock->delta();
    float frame_time_ms = DEFAULT_FRAME_TIME * 1000;
    if ( frame_time_ms - last_time < 0 ) {
      usleep(0);
    }
    else {
      usleep(frame_time_ms - last_time);
    }
    //lm.writeLog("Sleep Time: %f\n", frame_time_ms - last_time );

    //lm.writeLog("End of Loop # %d", counter);
    counter++;
  }
  if(game_over)
    shutDown();

};

void GameManager::run(int frame_time) {

  Clock *clock = new Clock();

  // game loop
  while(!game_over) {
    //inputManager.getInput();
    

    // objects update themselves
    WorldManager &world_manager = WorldManager::getInstance();
    EventStep *step = new EventStep();
    world_manager.onEvent(step);

    //graphicsManager.swapBuffers();
    int last_time = clock->delta();
    int frame_time_ms = frame_time * 1000;
    usleep(frame_time_ms - last_time);
  }
  if(game_over)
    shutDown();

};

void GameManager::setGameOver() {
  game_over = true;
};

void GameManager::shutDown() {
  LogManager &lm = LogManager::getInstance();
  GraphicsManager &grm = GraphicsManager::getInstance();
  InputManager &im = InputManager::getInstance();
  im.shutDown();
  grm.shutDown();
  lm.writeLog("Engine shut down");
  exit(-1);

};
