//
//GraphicsManager.cpp
//

#include "GraphicsManager.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "GameObjectListIterator.h"
#include "EventOut.h"
#include <stdio.h>


GraphicsManager::GraphicsManager() {
  window_horiz = 0;
  window_vert = 0;
};

GraphicsManager::GraphicsManager(GraphicsManager const&) {

};

void GraphicsManager::operator=(GraphicsManager const&) {

};

GraphicsManager::~GraphicsManager() {

};

GraphicsManager &GraphicsManager::getInstance() {
  static GraphicsManager instance;
  return instance;
};

int GraphicsManager::startUp(){
  LogManager &lm = LogManager::getInstance();
  initscr();
  if(has_colors()){
    start_color();
	init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
	init_pair(COLOR_RED, COLOR_RED, COLOR_BLACK);
	init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
	init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
	init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
	init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
	init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
  }
  else
    lm.writeLog("Terminal does not support color");
  getmaxyx(stdscr,window_vert,window_horiz);
  win1 = newwin(0, 0, 0, 0);
  leaveok(win1,true);
  current = 1;
  lm.writeLog("Graphics Manager started");
  return 0;
};

void GraphicsManager::shutDown(){
  LogManager &lm = LogManager::getInstance();
  endwin();
  lm.writeLog("Graphics Manager shut down");
};

int GraphicsManager::swapBuffers(){
  LogManager &lm = LogManager::getInstance();
  wrefresh(win1);
  werase(win1);
  return 0;
};

int GraphicsManager::drawCh(Position world_pos, char ch){
  LogManager &lm = LogManager::getInstance();
  /*if(world_pos.getX()>=0 && world_pos.getX()<=getHorizontal() &&
    world_pos.getY()>=0 && world_pos.getY()<=getVertical()){*/
  wattron(win1,A_BOLD);
  int x = world_pos.getX();
  int y = world_pos.getY();
 // lm.writeLog("drawing %c at %d, %d", ch, x, y);
  mvwaddch(win1,y,x,ch);
  wattroff(win1,A_BOLD);
  return 0;
 // }
 // else{
    WorldManager &wm = WorldManager::getInstance();
    EventOut out;
    GameObjectListIterator iter = wm.getAllObjects().createIterator();
    while(!iter.isDone()){
      if(iter.currentObj()->getPos().getX() == world_pos.getX() &&
        iter.currentObj()->getPos().getY() == world_pos.getY())
        out.setObj(iter.currentObj());
      iter.next();
    }
    lm.writeLog("Something left the world");
    wm.onEvent(&out);
 // }
};

int GraphicsManager::drawCh(Position world_pos, char ch, int color){
    LogManager &lm = LogManager::getInstance();
  /*if(world_pos.getX()>=0 && world_pos.getX()<=getHorizontal() &&
    world_pos.getY()>=0 && world_pos.getY()<=getVertical()){*/
  if(has_colors())
	wattron(win1,COLOR_PAIR(color));
  wattron(win1,A_BOLD);
  int x = world_pos.getX();
  int y = world_pos.getY();
  //lm.writeLog("drawing %c", ch);
  mvwaddch(win1,y,x,(chtype)ch);
  if(has_colors())
	wattroff(win1,COLOR_PAIR(color));
  wattroff(win1,A_BOLD);
  return 0;
 // }
 // else{
    WorldManager &wm = WorldManager::getInstance();
    EventOut out;
    GameObjectListIterator iter = wm.getAllObjects().createIterator();
    while(!iter.isDone()){
      if(iter.currentObj()->getPos().getX() == world_pos.getX() &&
        iter.currentObj()->getPos().getY() == world_pos.getY())
        out.setObj(iter.currentObj());
      iter.next();
    }
    lm.writeLog("Something left the world");
    wm.onEvent(&out);
  //}
};

int GraphicsManager::getHorizontal(){
  return window_horiz;
};

int GraphicsManager::getVertical(){
  return window_vert;
};

Position GraphicsManager::worldToScreen(Position world_pos){
  WorldManager &wm = WorldManager::getInstance();
  int view_x = wm.getView().getCorner().getX();
  int view_y = wm.getView().getCorner().getY();
  Position screen_pos( world_pos.getX() - view_x, 	world_pos.getY() - view_y);
  return screen_pos;
};

int GraphicsManager::drawFrame(Position world_pos, Frame frame, bool centered, int color) {
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("drawing a frame");
	int x_offset, y_offset;
	if(frame.getString() != "") {
		if(centered) {
			//lm.writeLog("Sprite is centered");
			y_offset = frame.getHeight()/2;
			x_offset = frame.getWidth()/2;
		}
		else {
			//lm.writeLog("Sprite is not centered");
			y_offset = 0;
			x_offset = 0;
		}
		string frame_str = frame.getString();
		/*lm.writeLog("Saved string to temp variable as %s", frame_str.c_str());
		lm.writeLog("Frame height is %d", frame.getHeight());
		lm.writeLog("Frame width is %d", frame.getWidth());*/
		for(int y = 0; y < frame.getHeight(); y++) {
			for(int x = 0; x < frame.getWidth(); x++) {
				//lm.writeLog("Into the for loop...");
				Position temp_pos(world_pos.getX() -x_offset + x, world_pos.getY() - y_offset + y);
				//lm.writeLog("created new position for %d, %d", temp_pos.getX(), temp_pos.getY());
				Position new_temp = worldToScreen(temp_pos);
				//lm.writeLog("Screen Position : %d, %d", new_temp.getX(), new_temp.getY());
				drawCh(new_temp, (char)frame_str[y * frame.getWidth() + x], color);
			}
		}
	}
};
