/*
* ResourceManager.cpp
*/

#include "ResourceManager.h"
#include "GraphicsManager.h"
#include "LogManager.h"

#include <iostream>
#include <fstream>
#include "Sprite.h"

ResourceManager::ResourceManager() {
	
};

ResourceManager::~ResourceManager() {
	
};

int readLineInt(ifstream *p_file, int line_number, const char *tag) {
	char temp[100];
	p_file->getline(temp, 100);
	string line = string(temp);
	if(!line.compare(tag)) {
		return -1;
	}
	else {
		return atoi(line.substr(strlen(tag) + 1).c_str());
	}
};

string readLineStr(ifstream *p_file, int line_number, const char *tag) {
	char temp[100];
	for(int i = 0; i <= line_number; i++) {
		p_file->getline(temp, 100);
	}
	string line = string(temp);
	if(!line.compare(tag)) {
		return "";
	}
	else {
		return line.substr(6);
	}
};

int make_color(string color) {
	if(color == "blue") {
		return COLOR_BLUE;
	}
	else if(color == "green") {
		return COLOR_GREEN;
	}
	else if(color == "magenta") {
		return COLOR_MAGENTA;
	}
	else if(color == "cyan") {
		return COLOR_CYAN;
	}
	else if(color == "yellow") {
		return COLOR_YELLOW;
	}
	else if(color == "red") {
		return COLOR_RED;
	}
};

Frame readFrame(ifstream *p_file, int line_number, int width, int height) {
	char temp[100];
	string line, frame_str;
	
	for(int i = 0; i < height; i++) {
		p_file->getline(temp, 100);
		line = string(temp);
		frame_str += line;
	}
	p_file->getline(temp, 100);
	line = string(temp);
	if(!line.compare("end")) {
		Frame frame;
		return frame;
	}
	Frame frame(width, height, frame_str);
	return frame;
};

int ResourceManager::loadSprite(string filename, string label) {
	ifstream *file;
	file->open(filename.c_str());
	int frames = readLineInt(file, 0, FRAMES_TOKEN);
	int height = readLineInt(file, 1, HEIGHT_TOKEN);
	int width = readLineInt(file, 2, WIDTH_TOKEN);
	int color = make_color(readLineStr(file, 3, COLOR_TOKEN));
	Sprite *new_sprite = new Sprite(frames);
	new_sprite->setHeight(height);
	new_sprite->setWidth(width);
	new_sprite->setColor(color);
	
	for(int i = 0; i < frames; i++) {
		new_sprite->addFrame(readFrame(file, 0, width, height)); 
	}
	return frames;
};

int ResourceManager::loadSprite(string sprite_str, int height, int width, int frame_num, int color, string label) {
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("Loading sprite...");
	Sprite *new_sprite = new Sprite(frame_num);
	new_sprite->setHeight(height);
	new_sprite->setWidth(width);
	new_sprite->setColor(color);
	new_sprite->setLabel(label);
	
	//lm.writeLog("Sprite's height = %d", new_sprite->getHeight());
	//lm.writeLog("Sprite's width = %d", new_sprite->getWidth());
	
	for(int i = 0; i < frame_num; i++) {
		Frame frame;
		frame.setWidth(width);
		frame.setHeight(height);
		//lm.writeLog("Frame's height = %d", frame.getHeight());
		//lm.writeLog("Frame's width = %d", frame.getWidth());
		string temp = sprite_str.substr(i * height * width, (height * width));
		frame.setString(temp);
		//lm.writeLog("Frame's string = %s", frame.getString().c_str());
		new_sprite->addFrame(frame);
	};
	
	sprite[sprite_count] = new_sprite;
	sprite_count++;
};

Sprite * ResourceManager::getSprite(string label) {
	LogManager &lm = LogManager::getInstance();
	for(int i = 0; i < sprite_count; i++) {
		//lm.writeLog("Checking %s against %s", label.c_str(), sprite[i]->getLabel().c_str());
		if(label == sprite[i]->getLabel()) {
			//lm.writeLog("Sprite Found at Index %d", i);
			return sprite[i];
		}
	}
	return NULL;
};

int ResourceManager::startUp() {
	sprite_count = 0;
	return -1;
};

void ResourceManager::shutDown() {

};
