//
//WorldManager.cpp
//

#include "WorldManager.h"
#include "LogManager.h"
#include "GraphicsManager.h"
#include "EventCollision.h"
#include "EventOut.h"
#include "EventOffscreen.h"
#include <stdio.h>

WorldManager::WorldManager(WorldManager const&) {

};

void WorldManager::operator=(WorldManager const&) {

};

WorldManager::WorldManager() {

};

WorldManager::~WorldManager() {

};

WorldManager &WorldManager::getInstance() {
  static WorldManager instance;
  return instance;
};

bool WorldManager::valueInRange(int value, int min, int max) {
  bool inRange = true;
  if(value<min)
    inRange = false;
  if(value>max)
    inRange = false;
  return inRange;
};

int WorldManager::startUp() {
  obj_id = 0;
  
};

void WorldManager::shutDown() {

};

int WorldManager::markForDelete(GameObject *p_go) {
  
  if(obj.find(p_go->getId())!=NULL){
    obj.remove(p_go);
    del.insert(p_go);
    return 0;
  }
  return -1;
};

GameObjectList WorldManager::getAllObjects(void) {
  return obj;
};

int WorldManager::insertObj(GameObject *p_go) {
	LogManager &lm = LogManager::getInstance();
  p_go->setId(obj_id);
  if(obj.find(p_go->getId()) == NULL){
    obj.insert(p_go);
    if(p_go->isSolid())
      registerInterest(p_go, "EVENT_COLLISION");
    obj_id++;
    return 0;
  }
  lm.writeLog("Error: Tried to add object already in list");
  return -1;
};

int WorldManager::removeObj(GameObject *p_go) {
  if(del.find(p_go->getId())!=NULL){
    del.remove(p_go);
    return 0;
  }
  LogManager &lm = LogManager::getInstance();
  lm.writeLog("ERROR: Tried to remove object not in list");
  return -1;
};

GameObject *WorldManager::findObj(int id) {
  if(obj.find(id)!=NULL)
    return obj.find(id);
  return del.find(id);
};

void WorldManager::update() {
  LogManager &lm = LogManager::getInstance();
  EventStep step;
  onEvent(&step);
  
  GameObjectListIterator iter = del.createIterator();
  iter.first();
  while(!iter.isDone()){
	GameObject *temp_go = iter.currentObj();
	delete temp_go;
	iter.next();
  };
  del.clear();
};

int WorldManager::moveObj(GameObject *p_go, Position where) {
  LogManager &lm = LogManager::getInstance();
	if(p_go->isSolid()) {
		GameObject *p_temp_go;
		p_temp_go = isCollision(p_go, where);
		if(p_temp_go) {
			EventCollision *first_e = new EventCollision(p_temp_go);
			EventCollision *second_e = new EventCollision(p_go);
			p_go->eventHandler(first_e);
			p_temp_go->eventHandler(second_e);
			return -1;
		}
	}
	
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	Position temp_pos = graph_man.worldToScreen(p_go->getPos());
	Position new_pos = graph_man.worldToScreen(where);
	bool on_screen = (0 <= temp_pos.getX() && temp_pos.getX() + p_go->getBox().getHorizontal()  < getView().getHorizontal()) &&
					 (0 <= temp_pos.getY() && temp_pos.getY() + p_go->getBox().getVertical() < getView().getVertical());
	
	bool off_screen = (new_pos.getX() < 0 || new_pos.getX() + p_go->getBox().getHorizontal() >= getView().getHorizontal()) ||
					  (new_pos.getY() < 0 || new_pos.getY() + p_go->getBox().getVertical() >= getView().getVertical());
	p_go->setPos(where);			  
	if(on_screen && off_screen) {
		EventOffscreen *off_screen = new EventOffscreen();
		p_go->eventHandler(off_screen);
	}
	bool on_world = (0 <= p_go->getPos().getX() && p_go->getPos().getX() + p_go->getBox().getHorizontal()  < getBoundary().getHorizontal()) &&
					 (0 <= p_go->getPos().getY() && p_go->getPos().getY() + p_go->getBox().getVertical() < getBoundary().getVertical());
	
	bool off_world = (where.getX() < 0 || where.getX() + p_go->getBox().getHorizontal() >= getBoundary().getHorizontal()) ||
					  (where.getY() < 0 || where.getY() + p_go->getBox().getVertical() >= getBoundary().getVertical());
	if(on_world && off_world) {
		EventOut *out = new EventOut();
		p_go->eventHandler(out);
	}
	return 0;
};

void WorldManager::pointDetect(GameObject *p_go, Position check_pos) {
    GameObjectListIterator iter = obj.createIterator();
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("Position is %d, %d", check_pos.getX(), check_pos.getY());
	 while(!iter.isDone()){
		if(p_go != iter.currentObj() && boxAndPoint(check_pos, iter.currentObj()->getBox())) {
			EventCollision *first_e = new EventCollision(p_go);
			iter.currentObj()->eventHandler(first_e);
		};
		iter.next();
	};
};

GameObject * WorldManager::objectPointDetect(GameObject *p_go, Position check_pos) {
	GameObjectListIterator iter = obj.createIterator();
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("Position is %d, %d", check_pos.getX(), check_pos.getY());
	 while(!iter.isDone()){
		if(p_go != iter.currentObj() && boxAndPoint(check_pos, iter.currentObj()->getBox())) {
			return iter.currentObj();
		};
		iter.next();
	};
	return NULL;
}

bool WorldManager::boxAndPoint(Position pos, Box box) {
	bool x_dir = pos.getX() >= box.getCorner().getX() && pos.getX() <= box.getCorner().getX() + box.getHorizontal() - 1;
	bool y_dir = pos.getY() >= box.getCorner().getY() && pos.getY() <= box.getCorner().getY() + box.getVertical() - 1;
	return x_dir && y_dir;
};

bool WorldManager::boxesIntersect(Box box1, Box box2) {
	int box1_left = box1.getCorner().getX();
	int box1_right = box1.getCorner().getX() + box1.getHorizontal() - 1;
	int box2_left = box2.getCorner().getX();
	int box2_right = box2.getCorner().getX() + box2.getHorizontal() - 1;
	
	bool x_overlap = (box2_left <= box1_left && box1_left <= box2_right) ||
					 (box1_left <= box2_left && box2_left <= box1_right);
					 
	int box1_top = box1.getCorner().getY();
	int box1_bottom = box1.getCorner().getY() + box1.getVertical() - 1;
	int box2_top = box2.getCorner().getY();
	int box2_bottom = box2.getCorner().getY() + box2.getVertical() - 1;
	
	bool y_overlap = (box2_top <= box1_top && box1_top <= box2_bottom) ||
					 (box1_top <= box2_top && box2_top <= box1_bottom);
	
	if(x_overlap && y_overlap) {
		return true;
	}
	else {
		return false;
	}
};

GameObject *WorldManager::isCollision(GameObject *p_go, Position where) {
  if(p_go->isSolid()){
	  LogManager &lm = LogManager::getInstance();
    GameObjectListIterator iter = obj.createIterator();
	//lm.writeLog("The moving Object is at %d, %d",p_go->getPos().getX(),p_go->getPos().getY());
    while(!iter.isDone()){
		Box where_box(where, p_go->getBox().getHorizontal(), p_go->getBox().getVertical());
      if(boxesIntersect(where_box, iter.currentObj()->getBox()) && (p_go->getId() != iter.currentObj()->getId())){
		  //lm.writeLog("The current object in the list is at %d, %d",iter.currentObj()->getPos().getX(),iter.currentObj()->getPos().getY());
        if(iter.currentObj()->isSolid() &&
          p_go->getAltitude() == iter.currentObj()->getAltitude()){
          //lm.writeLog("A collision ocurred at %d, %d",where.getX(),where.getY());
          return iter.currentObj();
        }
      }
      iter.next();
    }
  }
  return NULL;
};

int WorldManager::draw(){
	LogManager &lm = LogManager::getInstance();
  for(int alt = -MAX_ALTITUDE; alt<=MAX_ALTITUDE;alt++){
    GameObjectListIterator iter = obj.createIterator();
    while(!iter.isDone()){
      GameObject *current = iter.currentObj();
	  
      if(current->getAltitude()==alt)/* && current->getPos().getX() >= view.getCorner().getX()
        && current->getPos().getX() <= view.getCorner().getX() + view.getHorizontal()
        && current->getPos().getY() >= view.getCorner().getY()
        && current->getPos().getY() >= view.getCorner().getY() + view.getVertical())*/
        current->draw();
      iter.next();
    }
  }
  return 0;
};

Box WorldManager::getBoundary(){
  return boundary;
};

void WorldManager::setBoundary(Box new_boundary){
  boundary = new_boundary;
};

Box WorldManager::getView(){
  return view;
};

void WorldManager::setView(Box new_view){
  view = new_view;
};

void WorldManager::setViewPosition(Position view_pos){
  int x = view_pos.getX();
  if (x + view.getHorizontal() > boundary.getHorizontal())
    x = boundary.getHorizontal()-view.getHorizontal();
  if (x < 0)
   x = 0;
  int y = view_pos.getY();
  if (y + view.getVertical() > boundary.getVertical())
    y = boundary.getVertical()-view.getVertical();
  if (y < 0)
   y = 0;
  Position *new_corner = new Position(x,y);
  Box *new_view = new Box(*new_corner, view.getHorizontal(), view.getVertical());
  view = *new_view;
};
