///
/// worldManager.h
///

#ifndef __WORLD_MANAGER_H__
#define __WORLD_MANAGER_H__

#include "Manager.h"
#include "GameObjectList.h"
#include "GameObject.h"
#include "Position.h"
#include "EventStep.h"
#include "EventOut.h"
#include "Box.h"

#define MAX_ALTITUDE 2

class WorldManager : public Manager {

 private:
  int obj_id;
  WorldManager (WorldManager const&);
  void operator=(WorldManager const&);
  WorldManager();
  static WorldManager *instance;

 protected:
  GameObjectList obj;
  GameObjectList del;
  Box boundary;
  Box view;
  bool valueInRange(int value, int min, int max);

 public:
  ~WorldManager();
  static WorldManager &getInstance();
  int startUp();
  void shutDown();
  int markForDelete(GameObject *p_go);
  GameObjectList getAllObjects(void);
  int insertObj(GameObject *p_go);
  int removeObj(GameObject *p_go);
  GameObject *findObj(int id);
  void update();
  int moveObj(GameObject *p_go, Position where);
  GameObject *isCollision(GameObject *p_go, Position where);
  bool boxesIntersect(Box box1, Box box2);
  int draw();
  Box getBoundary();
  void setBoundary(Box new_boundary);
  Box getView();
  void setView(Box new_view);
  void setViewPosition(Position view_pos);
  void pointDetect(GameObject *p_go, Position check_pos);
  GameObject * objectPointDetect(GameObject *p_go, Position check_pos);
  bool boxAndPoint(Position pos, Box box);
};

#endif
