//
// FirstEnemy.cpp
//

#include "FirstEnemy.h"
#include "EnemyFactory.h"
#include "LifeDrop.h"
#include <stdlib.h>

FirstEnemy::FirstEnemy(Position init_pos) {
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	pos = init_pos;
	altitude = 0;
	dir = 0;
	ResourceManager &rm = ResourceManager::getInstance();
	p_sprite = rm.getSprite("firstEnemy");
	sprite_index = 0;
	sprite_slowdown = 60;
	sprite_slowdown_count = 0;
	box.setCorner(pos);
	box.setHorizontal(p_sprite->getWidth());
	box.setVertical(p_sprite->getHeight());
	shot_counter = 20;
	move_counter = 45;
	type = "FirstEnemy";
	setSolid(true);
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.addEnemy(this);
	health = 2;
	hurt_count = -1;
};

FirstEnemy::~FirstEnemy() {	
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.removeEnemy(this);
};

int FirstEnemy::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	ResourceManager &rm = ResourceManager::getInstance();
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		lm.writeLog("Enemy recieved a collision event");
		if(coll_e->getObj()->getType() == "bullet") {
			Bullet *bullet = static_cast<Bullet *>(coll_e->getObj());
			int damage = bullet->getDamage();
			health -= damage;
			hurt_count = 10;
			if(health <= 0) {
				WorldManager &wm = WorldManager::getInstance();
				wm.markForDelete(this);
				EnemyFactory &ef = EnemyFactory::getInstance();
				ef.spawnAmmo(pos);
			};
		}
		if(coll_e->getObj()->getType() == "rocket_explosion") {
			WorldManager &wm = WorldManager::getInstance();
			wm.markForDelete(this);
			EnemyFactory &ef = EnemyFactory::getInstance();
			ef.spawnAmmo(pos);
		}
		if(coll_e->getObj()->getType() == "SO") {
			health--;
			hurt_count = 10;
			if(health <= 0) {
				WorldManager &wm = WorldManager::getInstance();
				wm.markForDelete(this);
				EnemyFactory &ef = EnemyFactory::getInstance();
				ef.spawnAmmo(pos);
			};
		}
	}
	if(e->getType() == "EVENT_STEP") {
		if(hurt_count == 10) {
			p_sprite = rm.getSprite("firstEnemyhurt");
			hurt_count--;
		}
		else if(hurt_count > 0) {
			hurt_count--;
		}
		else if(hurt_count == 0) {
			p_sprite = rm.getSprite("firstEnemy");
			hurt_count--;
		}
		if(move_counter==0){
			move();
			move_counter = 45;
		}
		else{
			move_counter--;
		}
	}
	return 0;
};

void FirstEnemy::move() {
	WorldManager &wm = WorldManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	LogManager &lm = LogManager::getInstance();
	int p_x = pos.getX();
	int p_y = pos.getY();
    int hero_pos_x = ef.getHero().getPos().getX();
	int hero_pos_y = ef.getHero().getPos().getY();

    if(p_x == hero_pos_x && p_y > hero_pos_y)
		dir = 1;
	if(p_x < hero_pos_x && p_y > hero_pos_y)
		dir = 2;
	if(p_x < hero_pos_x && p_y == hero_pos_y)
		dir = 3;
	if(p_x < hero_pos_x && p_y < hero_pos_y)
		dir = 4;
	if(p_x == hero_pos_x && p_y < hero_pos_y)
		dir = 5;
	if(p_x > hero_pos_x && p_y < hero_pos_y)
		dir = 6;
	if(p_x > hero_pos_x && p_y == hero_pos_y)
		dir = 7;
    if(p_x > hero_pos_x && p_y > hero_pos_y)
		dir = 8;
	
	switch(dir){
	case 1:
		p_y--;
		break;
	case 2:
		p_x+=2;
		p_y--;
		break;
	case 3:
		p_x+=2;
		break;
	case 4:
		p_x+=2;
		p_y++;
		break;
	case 5:
		p_y++;
		break;
	case 6:
		p_x-=2;
		p_y++;
		break;
	case 7:
		p_x-=2;
		break;
	case 8:
		p_x-=2;
		p_y--;
		break;
	}

	Position new_pos(p_x,p_y);
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == new_pos.getY()) {
		box.setCorner(new_pos);
	};
};
