//
// Heatseeker.cpp
//

#include "Heatseeker.h"
#include "RocketExplosion.h"
#include "Engine/GraphicsManager.h"

Heatseeker::Heatseeker(Position init_pos, int dir) {
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("Heatseeker created");
	setPos(init_pos);
	distance = 10;
	counter = 2;
	init_counter = 2;
	direction = dir;
	target = NULL;
	setZone(direction);
};

Heatseeker::~Heatseeker() {

};

void Heatseeker::draw() {
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	Position new_pos = graph_man.worldToScreen(pos);
	graph_man.drawCh(new_pos, 'K');

};

int Heatseeker::eventHandler(Event *e) {
	WorldManager &wm = WorldManager::getInstance();
	if(e->getType() == "EVENT_STEP") {
		counter--;
		if(counter == 0) {
			counter = init_counter;
			if(!target) {
				search();
			}
			move();
		}
	}
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		if(coll_e->getObj()->getType() != "hero") {
			wm.markForDelete(this);
			new RocketExplosion(pos);
		}
	}
	if(e->getType() == "EVENT_OFFSCREEN") {
		wm.markForDelete(this);
	}
};

void Heatseeker::search() {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("searching...");
	for(int i = 0; i < 14; i++) {
		GameObject *p_go = wm.objectPointDetect(this, order[i]);
		if(p_go) {
			if(p_go->getType() == "FirstEnemy" || p_go->getType() == "SecondEnemy" || p_go->getType() == "BigEnemy") {
				lm.writeLog("found a target");
				target = p_go;
				return;
			};
		}
	}
};

void Heatseeker::move() {
	LogManager &lm = LogManager::getInstance();
	if(target) {
		lm.writeLog("has a target");
		Position target_pos = target->getPos();
		int x_diff = pos.getX() - target_pos.getX();
		int y_diff = pos.getY() - target_pos.getY();
		if(x_diff == 0 && y_diff > 0) {
			direction = UP;
		}
		else if(x_diff > 0 && y_diff > 0) {
			direction = UP_LEFT;
		}
		else if(x_diff > 0 && y_diff == 0) {
			direction = LEFT;
		}
		else if(x_diff > 0 && y_diff < 0) {
			direction = DOWN_LEFT;
		}
		else if(x_diff == 0 && y_diff < 0) {
			direction = DOWN;
		}
		else if(x_diff > 0 && y_diff < 0) {
			direction = DOWN_RIGHT;
		}
		else if(x_diff > 0 && y_diff == 0) {
			direction = RIGHT;
		}
		else if(x_diff > 0 && y_diff > 0) {
			direction = UP_RIGHT;
		}
	}
	int x = pos.getX();
	int y = pos.getY();
	if(direction == UP || direction == UP_RIGHT || direction == UP_LEFT) {
		y--;
	}
	else if(direction == DOWN || direction == DOWN_RIGHT || direction == DOWN_LEFT) {
		y++;
	}
	else if(direction == LEFT || direction == DOWN_LEFT || direction == UP_LEFT) {
		x = x-2;
	}
	else if(direction == RIGHT || direction == UP_RIGHT || direction == DOWN_RIGHT) {
		x = x+2;
	}
	WorldManager &wm = WorldManager::getInstance();
	Position new_pos(x,y);
	if(distance > 0) {
		distance--;
	};
	if(distance == 0) {
		wm.markForDelete(this);
		new RocketExplosion(pos);
	}
	if(x < 0 || y < 0 || x > wm.getBoundary().getHorizontal() || y > wm.getBoundary().getVertical()) {
		wm.markForDelete(this);
		return;		
	}
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == pos.getY()) {
		box.setCorner(new_pos);
		setZone(direction);
	}
};

void Heatseeker::setZone(int dir) {
	if(dir == UP) {
		Position first(pos.getX(), pos.getY() - 1);
		order[0] = first;
		Position second(pos.getX() - 1, pos.getY() - 1);
		order[1] = second;
		Position third(pos.getX() + 1, pos.getY() - 1);
		order[2] = third;
		Position fourth(pos.getX(), pos.getY() - 2);
		order[3] = fourth;
		Position fifth(pos.getX() - 1, pos.getY());
		order[4] = fifth;
		Position sixth(pos.getX() + 1, pos.getY());
		order[5] = sixth;
		Position seventh(pos.getX() - 1, pos.getY() - 2);
		order[6] = seventh;
		Position eigth(pos.getX() + 1, pos.getY() - 2);
		order[7] = eigth;
		Position ninth(pos.getX() - 2, pos.getY() - 2);
		order[8] = ninth;
		Position tenth(pos.getX() + 2, pos.getY() - 2);
		order[9] = tenth;
		Position eleventh(pos.getX() - 2, pos.getY() - 1);
		order[10] = eleventh;
		Position twelfth(pos.getX() + 2, pos.getY() - 1);
		order[11] = twelfth;
		Position thirteenth(pos.getX() - 2, pos.getY());
		order[12] = thirteenth;
		Position fourteenth(pos.getX() + 2, pos.getY());
		order[13] = fourteenth;
	}
	else if(dir == UP_RIGHT) {
		Position first(pos.getX() + 1, pos.getY() - 1);
		order[0] = first;
		Position second(pos.getX(), pos.getY() - 1);
		order[1] = second;
		Position third(pos.getX() + 1, pos.getY());
		order[2] = third;
		Position fourth(pos.getX() + 2, pos.getY() - 1);
		order[3] = fourth;
		Position fifth(pos.getX() + 3, pos.getY() - 2);
		order[4] = fifth;
		Position sixth(pos.getX() + 2, pos.getY() - 2);
		order[5] = sixth;
		Position seventh(pos.getX() - 3, pos.getY() - 1);
		order[6] = seventh;
		Position eigth(pos.getX() + 2, pos.getY());
		order[7] = eigth;
		Position ninth(pos.getX() + 1, pos.getY() - 2);
		order[8] = ninth;
		Position tenth(pos.getX() + 4, pos.getY() - 2);
		order[9] = tenth;
		Position eleventh(pos.getX() + 4, pos.getY() - 1);
		order[10] = eleventh;
		Position twelfth(pos.getX(), pos.getY() - 2);
		order[11] = twelfth;
		Position thirteenth(pos.getX() + 3, pos.getY());
		order[12] = thirteenth;
		Position fourteenth(pos.getX() + 4, pos.getY());
		order[13] = fourteenth;
	}
	else if(dir == RIGHT) {
		Position first(pos.getX() + 1, pos.getY());
		order[0] = first;
		Position second(pos.getX() + 1, pos.getY() - 1);
		order[1] = second;
		Position third(pos.getX() + 1, pos.getY() + 1);
		order[2] = third;
		Position fourth(pos.getX() + 2, pos.getY());
		order[3] = fourth;
		Position fifth(pos.getX(), pos.getY() - 1);
		order[4] = fifth;
		Position sixth(pos.getX(), pos.getY() + 1);
		order[5] = sixth;
		Position seventh(pos.getX() + 3, pos.getY());
		order[6] = seventh;
		Position eigth(pos.getX() + 2, pos.getY() - 1);
		order[7] = eigth;
		Position ninth(pos.getX() + 2, pos.getY() + 1);
		order[8] = ninth;
		Position tenth(pos.getX() + 4, pos.getY());
		order[9] = tenth;
		Position eleventh(pos.getX() + 3, pos.getY() - 1);
		order[10] = eleventh;
		Position twelfth(pos.getX() + 3, pos.getY() + 1);
		order[11] = twelfth;
		Position thirteenth(pos.getX() + 4, pos.getY() - 1);
		order[12] = thirteenth;
		Position fourteenth(pos.getX() + 4, pos.getY() + 1);
		order[13] = fourteenth;
	}
	else if(dir == DOWN_RIGHT) {
		Position first(pos.getX() + 1, pos.getY() + 1);
		order[0] = first;
		Position second(pos.getX() + 1, pos.getY());
		order[1] = second;
		Position third(pos.getX(), pos.getY() + 1);
		order[2] = third;
		Position fourth(pos.getX() + 2, pos.getY() + 1);
		order[3] = fourth;
		Position fifth(pos.getX() + 3, pos.getY() + 2);
		order[4] = fifth;
		Position sixth(pos.getX() + 3, pos.getY() + 1);
		order[5] = sixth;
		Position seventh(pos.getX() + 2, pos.getY() + 2);
		order[6] = seventh;
		Position eigth(pos.getX() + 2, pos.getY());
		order[7] = eigth;
		Position ninth(pos.getX() + 1, pos.getY() + 2);
		order[8] = ninth;
		Position tenth(pos.getX() + 4, pos.getY() + 2);
		order[9] = tenth;
		Position eleventh(pos.getX() + 4, pos.getY() + 1);
		order[10] = eleventh;
		Position twelfth(pos.getX(), pos.getY() + 2);
		order[11] = twelfth;
		Position thirteenth(pos.getX() + 3, pos.getY());
		order[12] = thirteenth;
		Position fourteenth(pos.getX() + 4, pos.getY());
		order[13] = fourteenth;
	}
	else if(dir == DOWN) {
		Position first(pos.getX(), pos.getY() + 1);
		order[0] = first;
		Position second(pos.getX() + 1, pos.getY() + 1);
		order[1] = second;
		Position third(pos.getX() - 1, pos.getY() + 1);
		order[2] = third;
		Position fourth(pos.getX(), pos.getY() + 2);
		order[3] = fourth;
		Position fifth(pos.getX() + 1, pos.getY());
		order[4] = fifth;
		Position sixth(pos.getX() - 1, pos.getY());
		order[5] = sixth;
		Position seventh(pos.getX() + 1, pos.getY() + 2);
		order[6] = seventh;
		Position eigth(pos.getX() - 1, pos.getY() + 2);
		order[7] = eigth;
		Position ninth(pos.getX() + 2, pos.getY() + 2);
		order[8] = ninth;
		Position tenth(pos.getX() - 2, pos.getY() + 2);
		order[9] = tenth;
		Position eleventh(pos.getX() + 2, pos.getY() + 1);
		order[10] = eleventh;
		Position twelfth(pos.getX() - 2, pos.getY() + 1);
		order[11] = twelfth;
		Position thirteenth(pos.getX() + 2, pos.getY());
		order[12] = thirteenth;
		Position fourteenth(pos.getX() - 2, pos.getY());
		order[13] = fourteenth;
	}
	else if(dir == DOWN_LEFT) {
		Position first(pos.getX() - 1, pos.getY() + 1);
		order[0] = first;
		Position second(pos.getX(), pos.getY() + 1);
		order[1] = second;
		Position third(pos.getX() - 1, pos.getY());
		order[2] = third;
		Position fourth(pos.getX() - 2, pos.getY() + 1);
		order[3] = fourth;
		Position fifth(pos.getX() - 3, pos.getY() + 2);
		order[4] = fifth;
		Position sixth(pos.getX() - 2, pos.getY() + 2);
		order[5] = sixth;
		Position seventh(pos.getX() - 3, pos.getY() + 1);
		order[6] = seventh;
		Position eigth(pos.getX() - 2, pos.getY());
		order[7] = eigth;
		Position ninth(pos.getX() - 1, pos.getY() + 2);
		order[8] = ninth;
		Position tenth(pos.getX() - 4, pos.getY() + 2);
		order[9] = tenth;
		Position eleventh(pos.getX() - 4, pos.getY() + 1);
		order[10] = eleventh;
		Position twelfth(pos.getX(), pos.getY() + 2);
		order[11] = twelfth;
		Position thirteenth(pos.getX() - 3, pos.getY());
		order[12] = thirteenth;
		Position fourteenth(pos.getX() - 4, pos.getY());
		order[13] = fourteenth;
	}
	else if(dir == LEFT) {
		Position first(pos.getX() - 1, pos.getY());
		order[0] = first;
		Position second(pos.getX() - 1, pos.getY() + 1);
		order[1] = second;
		Position third(pos.getX() - 1, pos.getY() - 1);
		order[2] = third;
		Position fourth(pos.getX() - 2, pos.getY());
		order[3] = fourth;
		Position fifth(pos.getX(), pos.getY() + 1);
		order[4] = fifth;
		Position sixth(pos.getX(), pos.getY() - 1);
		order[5] = sixth;
		Position seventh(pos.getX() - 3, pos.getY());
		order[6] = seventh;
		Position eigth(pos.getX() - 2, pos.getY() + 1);
		order[7] = eigth;
		Position ninth(pos.getX() - 2, pos.getY() - 1);
		order[8] = ninth;
		Position tenth(pos.getX() - 4, pos.getY());
		order[9] = tenth;
		Position eleventh(pos.getX() - 3, pos.getY() + 1);
		order[10] = eleventh;
		Position twelfth(pos.getX() - 3, pos.getY() - 1);
		order[11] = twelfth;
		Position thirteenth(pos.getX() - 4, pos.getY() + 1);
		order[12] = thirteenth;
		Position fourteenth(pos.getX() - 4, pos.getY() - 1);
		order[13] = fourteenth;
	}
	else if(dir == UP_LEFT) {
		Position first(pos.getX() - 1, pos.getY() + 1);
		order[0] = first;
		Position second(pos.getX() - 1, pos.getY());
		order[1] = second;
		Position third(pos.getX(), pos.getY() + 1);
		order[2] = third;
		Position fourth(pos.getX() - 2, pos.getY() + 1);
		order[3] = fourth;
		Position fifth(pos.getX() - 3, pos.getY() + 2);
		order[4] = fifth;
		Position sixth(pos.getX() - 3, pos.getY() + 1);
		order[5] = sixth;
		Position seventh(pos.getX() - 2, pos.getY() + 2);
		order[6] = seventh;
		Position eigth(pos.getX() - 2, pos.getY());
		order[7] = eigth;
		Position ninth(pos.getX() - 1, pos.getY() + 2);
		order[8] = ninth;
		Position tenth(pos.getX() - 4, pos.getY() + 2);
		order[9] = tenth;
		Position eleventh(pos.getX() - 4, pos.getY() + 1);
		order[10] = eleventh;
		Position twelfth(pos.getX(), pos.getY() + 2);
		order[11] = twelfth;
		Position thirteenth(pos.getX() - 3, pos.getY());
		order[12] = thirteenth;
		Position fourteenth(pos.getX() - 4, pos.getY());
		order[13] = fourteenth;
	}
};
