//
// Hero.cpp
//

#include "Hero.h"
#include "StandardBullet.h"
#include "DirectionalBullet.h"
#include "ScatterBullet.h"
#include "RailGun.h"
#include "Rocket.h"
#include "Heatseeker.h"
#include "AmmoDrop.h"
#include "EventLife.h"
#include "EventAmmo.h"
#include "EventGunChange.h"
#include "EnemyFactory.h"

Hero::Hero() {
	InputManager &im = InputManager::getInstance();
	WorldManager &wm = WorldManager::getInstance();
	im.registerInterest(this, "EVENT_KEYBOARD");
	wm.registerInterest(this, "EVENT_STEP");
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	int new_x = wm.getView().getCorner().getX() + wm.getView().getHorizontal()/2;
	int new_y = wm.getView().getCorner().getY() + wm.getView().getVertical()/2;
	pos.setX(new_x -2);
	pos.setY(new_y - 1);
	ResourceManager &rm = ResourceManager::getInstance();
	p_sprite = rm.getSprite("hero");
	sprite_index = 0;
	sprite_slowdown = 60;
	sprite_slowdown_count = 0;
	box.setCorner(pos);
	box.setHorizontal(p_sprite->getWidth());
	box.setVertical(p_sprite->getHeight());
	shot_counter = 20;
	type = "hero";
	setSolid(true);
	move_counter = 5;
	lives = 3;
	gun_type[STANDARD] = "standard";
	ammo_count[STANDARD] = -1;
	current_gun = STANDARD;
	gun_type[SECOND] = "second";
	ammo_count[SECOND] = 8;
	gun_type[SCATTER] = "scatter";
	ammo_count[SCATTER] = 4;
	gun_type[RAIL] = "rail";
	ammo_count[RAIL] = 3;
	gun_type[ROCKET] = "rocket";
	ammo_count[ROCKET] = 2;
};

Hero::~Hero() {	
	InputManager &im = InputManager::getInstance();
	im.unregisterInterest(this, "EVENT_KEYBOARD");
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
};

int Hero::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("Hero: Event Handler Called");
	if(e->getType() == "EVENT_KEYBOARD") {
		EventKeyboard *p_keyboard_event = static_cast <EventKeyboard *> (e);
		keyProcess(p_keyboard_event);
	}
	if(e->getType() == "EVENT_OUT") {
		out();
	}
	if(e->getType() == "EVENT_STEP") {
		if(shot_counter > 0) {
			shot_counter--;
		}
		if(move_counter > 0) {
			move_counter--;
		}
	}
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		//lm.writeLog("Hero recieved a collision event");
		if(coll_e->getObj()->getType() == "ammo_drop") {
			//lm.writeLog("nope this one");
			addAmmo(coll_e);
		}
		else if(coll_e->getObj()->getType() == "life_drop") {
			if(lives < 9) {
				lives++;
				WorldManager &wm = WorldManager::getInstance();
				EventLife *new_e = new EventLife(lives);
				wm.onEvent(new_e);
			};
		}
		else if(coll_e->getObj()->getType() != "bullet" && coll_e->getObj()->getType() != "rocket_explosion") {
			//lm.writeLog("this one");
			lives--;
			WorldManager &wm = WorldManager::getInstance();
			EventLife *new_e = new EventLife(lives);
			wm.onEvent(new_e);
			if(lives <= 0) {
				GameManager &gm = GameManager::getInstance();
				gm.setGameOver();
			}
		}
	}
	return 0;
};

void Hero::keyProcess(EventKeyboard *e) {
	LogManager &lm = LogManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	WorldManager &wm = WorldManager::getInstance();
	//lm.writeLog("keyboard event recieved");
	switch(e->getKey()) {
		case 'w':
			move(UP);
			break;
		case 'a':
			move(LEFT);
			break;
		case 's':
			move(DOWN);
			break;
		case 'd':
			move(RIGHT);
			break;
		case KEY_UP:
			shoot(UP);
			break;
		case KEY_LEFT:
			shoot(LEFT);
			break;
		case KEY_DOWN:
			shoot(DOWN);
			break;
		case KEY_RIGHT:
			shoot(RIGHT);
			break;
		case 27:
			GameManager::getInstance().setGameOver();
			break;
		case '1':
			current_gun = STANDARD;
			if(shot_counter > 20) {
				shot_counter = 20;
			}
			else {
				shot_counter = 20 - shot_counter;
			};
			//lm.writeLog("Counter is now %d", shot_counter);
			if(true) {
				EventGunChange *e_gun = new EventGunChange(gun_type[current_gun], ammo_count[current_gun]);
				wm.onEvent(e_gun);
			};
			break;
		case '2':
			current_gun = SECOND;
			if(shot_counter > 40) {
				shot_counter = 40;
			}
			else {
				shot_counter = 40 - shot_counter;
			}
			//lm.writeLog("Counter is now %d", shot_counter);
			if(true) {
				EventGunChange *e_gun = new EventGunChange(gun_type[current_gun], ammo_count[current_gun]);
				wm.onEvent(e_gun);
			};
			break;
		case '3':
			if(ef.getLevel() >=2) {
				current_gun = SCATTER;
				if(shot_counter > 30) {
					shot_counter = 30;
				}
				else {
					shot_counter = 30 - shot_counter;
				}
				if(true) {
					EventGunChange *e_gun = new EventGunChange(gun_type[current_gun], ammo_count[current_gun]);
					wm.onEvent(e_gun);
				}
			};
			break;
		case '4':
			if(ef.getLevel() >=2) {
				current_gun = RAIL;
				if(shot_counter > 50) {
					shot_counter = 50;
				}
				else {
					shot_counter = 50 - shot_counter;
				}
				if(true) {
					EventGunChange *e_gun = new EventGunChange(gun_type[current_gun], ammo_count[current_gun]);
					wm.onEvent(e_gun);
				}
			};
			break;
		case '5':
			if(ef.getLevel() >=3) {
				current_gun = ROCKET;
				if(shot_counter > 60) {
					shot_counter = 60;
				}
				else {
					shot_counter = 60 - shot_counter;
				}
				if(true) {
					EventGunChange *e_gun = new EventGunChange(gun_type[current_gun], ammo_count[current_gun]);
					wm.onEvent(e_gun);
				}
			};
			break;
	};
}

void Hero::move(int direction) {
	if(move_counter == 0) {
		move_counter = 5;
		WorldManager &wm = WorldManager::getInstance();
		int x = wm.getView().getCorner().getX();
		int y = wm.getView().getCorner().getY();
		int p_x = pos.getX();
		int p_y = pos.getY();
		if(direction == UP) {
			if(p_y > 0) {
				p_y--;
				if(p_y <= (wm.getBoundary().getVertical() - wm.getView().getVertical()/2) - 1) {
					y--;
				};
			};
		}
		else if(direction == DOWN) {
			if(p_y + 2 < wm.getBoundary().getVertical() - 1) {
				p_y++;
				if(p_y >= wm.getView().getVertical()/2 + 1) {
					y++;
				};
			};
		}
		else if(direction == RIGHT) {
			if(p_x + 2 < wm.getBoundary().getHorizontal() - 1) {
				p_x = p_x + 2;
				if(p_x >= wm.getView().getHorizontal()/2 + 1) {
					x = x + 2;
				};
			};
		}
		else if(direction == LEFT) {
			if(p_x > 1) {
				p_x = p_x - 2;
				if(p_x <= (wm.getBoundary().getHorizontal() - wm.getView().getHorizontal()/2) - 1) {
					x = x -2;
				};
			};
		};
		Position new_pos(x,y);
		Position new_p_pos(p_x,p_y);
		wm.moveObj(this, new_p_pos);
		if(pos.getX() == new_p_pos.getX() && pos.getY() == new_p_pos.getY()) {
			wm.setViewPosition(new_pos);
			box.setCorner(new_p_pos);
		}
	}
};

void Hero::out() {
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("Hero recieved an out event");
};

void Hero::shoot(int new_direction) {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	if(shot_counter == 0) {
		if(current_gun == STANDARD) {
			Bullet *temp_bullet;
			Position bullet_pos(pos.getX(), pos.getY());
			Position bullet_init(pos.getX(), pos.getY());
			if(new_direction == UP) {
				bullet_pos.setY(pos.getY() - 1);
				bullet_pos.setX(pos.getX() + 1);
				bullet_init.setY(pos.getY());
				bullet_init.setX(pos.getX() + 1);
			}
			else if(new_direction == DOWN) {
				bullet_pos.setY(pos.getY() + 3);
				bullet_pos.setX(pos.getX() + 1);
				bullet_init.setY(pos.getY() + 2);
				bullet_init.setX(pos.getX() + 1);
			}
			else if(new_direction == LEFT) {
				bullet_pos.setX(pos.getX() - 1);
				bullet_pos.setY(pos.getY() + 1);
				bullet_init.setX(pos.getX());
				bullet_init.setY(pos.getY() + 1);
			}
			else if(new_direction == RIGHT) {
				bullet_pos.setX(pos.getX() + 3);
				bullet_pos.setY(pos.getY() + 1);
				bullet_init.setX(pos.getX() + 2);
				bullet_init.setY(pos.getY() + 1);
			};
			temp_bullet = new StandardBullet(new_direction, bullet_init);
			shot_counter = 20;
			wm.moveObj(temp_bullet, bullet_pos);
		}
		else if(current_gun == SECOND) {
			if(ammo_count[current_gun] > 0) {
				shootDirectional();
			};
		}
		else if(current_gun == SCATTER) {
			if(ammo_count[current_gun] > 0) {
				shootScatter(new_direction);
			}
		}
		else if(current_gun == RAIL) {
			if(ammo_count[current_gun] > 0) {
				shootRailGun(new_direction);
			}
		}
		else if(current_gun == ROCKET) {
			if(ammo_count[current_gun] > 0) {
				Bullet *temp_bullet;
				Position bullet_pos(pos.getX(), pos.getY());
				Position bullet_init(pos.getX(), pos.getY());
				if(new_direction == UP) {
					bullet_pos.setY(pos.getY() - 1);
					bullet_pos.setX(pos.getX() + 1);
					bullet_init.setY(pos.getY());
					bullet_init.setX(pos.getX() + 1);
				}
				else if(new_direction == DOWN) {
					bullet_pos.setY(pos.getY() + 3);
					bullet_pos.setX(pos.getX() + 1);
					bullet_init.setY(pos.getY() + 2);
					bullet_init.setX(pos.getX() + 1);
				}
				else if(new_direction == LEFT) {
					bullet_pos.setX(pos.getX() - 1);
					bullet_pos.setY(pos.getY() + 1);
					bullet_init.setX(pos.getX());
					bullet_init.setY(pos.getY() + 1);
				}
				else if(new_direction == RIGHT) {
					bullet_pos.setX(pos.getX() + 3);
					bullet_pos.setY(pos.getY() + 1);
					bullet_init.setX(pos.getX() + 2);
					bullet_init.setY(pos.getY() + 1);
				};
				ammo_count[current_gun]--;
				temp_bullet = new Rocket(bullet_init, new_direction);
				shot_counter = 60;
				EventAmmo *e = new EventAmmo(gun_type[current_gun], ammo_count[current_gun]);
				wm.onEvent(e);
				wm.moveObj(temp_bullet, bullet_pos);
			}
		}
	}
};

void Hero::addAmmo(EventCollision *e) {
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("adding ammo...");
	GameObject *temp = e->getObj();
	//lm.writeLog("got object");
	AmmoDrop *drop = static_cast <AmmoDrop *> (temp);
	//lm.writeLog("cast as drop");
	for(int i = 0; i < MAX_GUNS; i++) {
		lm.writeLog("loop %s", gun_type[i].c_str());
		if(gun_type[i] == drop->getAmmoType()) {
			//lm.writeLog("ammo match");
			if(ammo_count[i] >= 0) {
				int ammo = ammo_count[i] + drop->getAmmo();
				if(ammo > 99) {
					ammo = 99;
				}
				ammo_count[i] = ammo;
				EventAmmo *e_ammo = new EventAmmo(gun_type[i], ammo_count[i]);
				WorldManager &wm = WorldManager::getInstance();
				wm.onEvent(e_ammo);
				return;
			}
		}
	};
};

void Hero::shootDirectional() {
	WorldManager &wm = WorldManager::getInstance();
	Position up_pos(pos.getX() + 1, pos.getY());
	Position new_up_pos(pos.getX() + 1, pos.getY() - 1);
	Bullet *up_bullet = new DirectionalBullet(UP, up_pos);
	wm.moveObj(up_bullet, new_up_pos);
	
	Position down_pos(pos.getX() + 1, pos.getY() + 2);
	Position new_down_pos(pos.getX() + 1, pos.getY() + 3);
	Bullet *down_bullet = new DirectionalBullet(DOWN, down_pos);
	wm.moveObj(down_bullet, new_down_pos);
	
	Position left_pos(pos.getX(), pos.getY() + 1);
	Position new_left_pos(pos.getX() - 1, pos.getY() + 1);
	Bullet *left_bullet = new DirectionalBullet(LEFT, left_pos);
	wm.moveObj(left_bullet, new_left_pos);
	
	Position right_pos(pos.getX() + 2, pos.getY() + 1);
	Position new_right_pos(pos.getX() + 3, pos.getY() + 1);
	Bullet *right_bullet = new DirectionalBullet(RIGHT, right_pos);
	wm.moveObj(right_bullet, new_right_pos);
	
	Position up_left_pos(pos.getX(), pos.getY());
	Position new_up_left_pos(pos.getX() - 1, pos.getY() - 1);
	Bullet *up_left_bullet = new DirectionalBullet(UP_LEFT, up_left_pos);
	wm.moveObj(up_left_bullet, new_up_left_pos);
	
	Position up_right_pos(pos.getX() + 2, pos.getY());
	Position new_up_right_pos(pos.getX() + 3, pos.getY() - 1);
	Bullet *up_right_bullet = new DirectionalBullet(UP_RIGHT, up_right_pos);
	wm.moveObj(up_right_bullet, new_up_right_pos);
	
	Position down_right_pos(pos.getX() + 2, pos.getY() + 2);
	Position new_down_right_pos(pos.getX() + 3, pos.getY() + 3);
	Bullet *down_right_bullet = new DirectionalBullet(DOWN_RIGHT, down_right_pos);
	wm.moveObj(down_right_bullet, new_down_right_pos);
	
	Position down_left_pos(pos.getX(), pos.getY() + 2);
	Position new_down_left_pos(pos.getX() - 1, pos.getY() + 3);
	Bullet *down_left_bullet = new DirectionalBullet(DOWN_LEFT, down_left_pos);
	wm.moveObj(down_left_bullet, new_down_left_pos);
	
	shot_counter = 40;
	ammo_count[current_gun]--;
	EventAmmo *e = new EventAmmo(gun_type[current_gun], ammo_count[current_gun]);
	wm.onEvent(e);
};

void Hero::shootScatter(int dir) {
	WorldManager &wm = WorldManager::getInstance();
	Position zeroth_pos, first_pos, second_pos, third_pos, fourth_pos, new_zeroth, new_first, new_second, new_third, new_fourth;
	if(dir == UP) {
		zeroth_pos.setX(pos.getX());
		zeroth_pos.setY(pos.getY());
		new_zeroth.setX(pos.getX() - 1);
		new_zeroth.setY(pos.getY() - 1);
		
		first_pos.setX(pos.getX());
		first_pos.setY(pos.getY());
		new_first.setX(pos.getX());
		new_first.setY(pos.getY() - 1);
		
		second_pos.setX(pos.getX() + 1);
		second_pos.setY(pos.getY());
		new_second.setX(pos.getX() + 1);
		new_second.setY(pos.getY() - 1);
		
		third_pos.setX(pos.getX() + 2);
		third_pos.setY(pos.getY());
		new_third.setX(pos.getX() + 2);
		new_third.setY(pos.getY() - 1);
		
		fourth_pos.setX(pos.getX() + 2);
		fourth_pos.setY(pos.getY());
		new_fourth.setX(pos.getX() + 3);
		new_fourth.setY(pos.getY() - 1);
	}
	else if(dir == DOWN) {
		zeroth_pos.setX(pos.getX());
		zeroth_pos.setY(pos.getY() + 2);
		new_zeroth.setX(pos.getX() - 1);
		new_zeroth.setY(pos.getY() + 3);
		
		first_pos.setX(pos.getX());
		first_pos.setY(pos.getY() + 2);
		new_first.setX(pos.getX());
		new_first.setY(pos.getY() + 3);
		
		second_pos.setX(pos.getX() + 1);
		second_pos.setY(pos.getY() + 2);
		new_second.setX(pos.getX() + 1);
		new_second.setY(pos.getY() + 3);
		
		third_pos.setX(pos.getX() + 2);
		third_pos.setY(pos.getY() + 2);
		new_third.setX(pos.getX() + 2);
		new_third.setY(pos.getY() + 3);
		
		fourth_pos.setX(pos.getX() + 2);
		fourth_pos.setY(pos.getY() + 2);
		new_fourth.setX(pos.getX() + 3);
		new_fourth.setY(pos.getY() + 3);
	}
	else if(dir == LEFT) {
		zeroth_pos.setX(pos.getX());
		zeroth_pos.setY(pos.getY());
		new_zeroth.setX(pos.getX() - 1);
		new_zeroth.setY(pos.getY() - 1);
		
		first_pos.setX(pos.getX());
		first_pos.setY(pos.getY());
		new_first.setX(pos.getX() - 1);
		new_first.setY(pos.getY());
		
		second_pos.setX(pos.getX());
		second_pos.setY(pos.getY() + 1);
		new_second.setX(pos.getX() - 1);
		new_second.setY(pos.getY() + 1);
		
		third_pos.setX(pos.getX());
		third_pos.setY(pos.getY() + 2);
		new_third.setX(pos.getX() - 1);
		new_third.setY(pos.getY() + 2);
		
		fourth_pos.setX(pos.getX());
		fourth_pos.setY(pos.getY() + 2);
		new_fourth.setX(pos.getX() - 1);
		new_fourth.setY(pos.getY() + 3);
	}
	else if(dir == RIGHT) {
		zeroth_pos.setX(pos.getX() + 2);
		zeroth_pos.setY(pos.getY());
		new_zeroth.setX(pos.getX() + 3);
		new_zeroth.setY(pos.getY() - 1);
		
		first_pos.setX(pos.getX() + 2);
		first_pos.setY(pos.getY());
		new_first.setX(pos.getX() + 3);
		new_first.setY(pos.getY());
		
		second_pos.setX(pos.getX() + 2);
		second_pos.setY(pos.getY() + 1);
		new_second.setX(pos.getX() + 3);
		new_second.setY(pos.getY() + 1);
		
		third_pos.setX(pos.getX() + 2);
		third_pos.setY(pos.getY() + 2);
		new_third.setX(pos.getX() + 3);
		new_third.setY(pos.getY() + 2);
		
		fourth_pos.setX(pos.getX() + 2);
		fourth_pos.setY(pos.getY() + 2);
		new_fourth.setX(pos.getX() + 3);
		new_fourth.setY(pos.getY() + 3);
	}
	
	Bullet *zeroth_bullet = new ScatterBullet(dir, zeroth_pos);
	wm.moveObj(zeroth_bullet, new_zeroth);
	
	Bullet *first_bullet = new ScatterBullet(dir, first_pos);
	wm.moveObj(first_bullet, new_first);
	
	Bullet *second_bullet = new ScatterBullet(dir, second_pos);
	wm.moveObj(second_bullet, new_second);
	
	Bullet *third_bullet = new ScatterBullet(dir, third_pos);
	wm.moveObj(third_bullet, new_third);
	
	Bullet *fourth_bullet = new ScatterBullet(dir, fourth_pos);
	wm.moveObj(fourth_bullet, new_fourth);
	
	shot_counter = 30;
	ammo_count[current_gun]--;
	EventAmmo *e = new EventAmmo(gun_type[current_gun], ammo_count[current_gun]);
	wm.onEvent(e);
};

void Hero::shootRailGun(int dir) {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	//lm.writeLog("It reached this function");
	Position new_pos(pos.getX(), pos.getY());
	Bullet *shot;
	if(dir == LEFT) {
		//lm.writeLog("left");
		new_pos.setX(pos.getX()-1);
		new_pos.setY(pos.getY()+1);
		shot = new RailGun(new_pos, dir);
		//lm.writeLog("hero's pos is %d, %d", pos.getX(), pos.getY());
	}
	else if(dir == RIGHT) {
		//lm.writeLog("right");
		new_pos.setX(pos.getX()+3);
		new_pos.setY(pos.getY()+1);
		shot = new RailGun(new_pos, dir);
	}
	else if(dir == UP) {
		//lm.writeLog("up");
		new_pos.setX(pos.getX()+1);
		new_pos.setY(pos.getY()-1);
		shot = new RailGun(new_pos, dir);
	}
	else if(dir == DOWN) {
		//lm.writeLog("down");
		new_pos.setX(pos.getX()+1);
		new_pos.setY(pos.getY()+3);
		shot = new RailGun(new_pos, dir);
	};
	wm.moveObj(shot, new_pos);
	
	shot_counter = 50;
	ammo_count[current_gun]--;
	EventAmmo *e = new EventAmmo(gun_type[current_gun], ammo_count[current_gun]);
	wm.onEvent(e);
};
