//
//MovingObstacle.cpp
//

#include "MovingObstacle.h"
#include "Engine/WorldManager.h"
#include "Engine/ResourceManager.h"
#include "Engine/EventCollision.h"
#include "Engine/Box.h"
#include "EnemyFactory.h"
#include "LifeDrop.h"
#include <stdlib.h>

MovingObstacle::MovingObstacle(Position init_pos, int init_direction){
	ResourceManager &rm = ResourceManager::getInstance();
	setSprite(rm.getSprite("MO"));
	pos = init_pos;
	direction = init_direction;
	setSolid(true);
	type = "MO";
	sprite_slowdown = 1;
	slowdown = 1;
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	Box *new_box = new Box(pos,1,1);
	setBox(*new_box);
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.addEnemy(this);
};

MovingObstacle::~MovingObstacle(){
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.removeEnemy(this);
};

int MovingObstacle::eventHandler(Event *e){
	WorldManager &wm = WorldManager::getInstance();
	if(e->getType() == "EVENT_COLLISION"){
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		if(coll_e->getObj()->getType() == "bullet" || coll_e->getObj()->getType() == "rocket_explosion") {
			wm.markForDelete(this);
			EnemyFactory &ef = EnemyFactory::getInstance();
			ef.spawnAmmo(pos);
		}
		else {
			wm.markForDelete(this);
		}
	}
	if(e->getType() == "EVENT_STEP"){
		if(slowdown == 1){
			move();
			slowdown = 3;
		}
		else{
			slowdown--;
		}
	}
	if(e->getType() == "EVENT_OUT"){
		out();
	}
	return 0;
};

void MovingObstacle::move(){
	WorldManager &wm = WorldManager::getInstance();
	Position new_pos(0,0);
	switch(direction){
	case 1:
		new_pos.setX(pos.getX());
		new_pos.setY(pos.getY()-1);
		wm.moveObj(this,new_pos);
		break;
	case 2:
		new_pos.setX(pos.getX()+1);
		new_pos.setY(pos.getY()-1);
		wm.moveObj(this,new_pos);
		break;
	case 3:
		new_pos.setX(pos.getX()+1);
		new_pos.setY(pos.getY());
		wm.moveObj(this,new_pos);
		break;
	case 4:
		new_pos.setX(pos.getX()+1);
		new_pos.setY(pos.getY()+1);
		wm.moveObj(this,new_pos);
		break;
	case 5:
		new_pos.setX(pos.getX());
		new_pos.setY(pos.getY()+1);
		wm.moveObj(this,new_pos);
		break;
	case 6:
		new_pos.setX(pos.getX()-1);
		new_pos.setY(pos.getY()+1);
		wm.moveObj(this,new_pos);
		break;
	case 7:
		new_pos.setX(pos.getX()-1);
		new_pos.setY(pos.getY());
		wm.moveObj(this,new_pos);
		break;
	case 8:
		new_pos.setX(pos.getX()-1);
		new_pos.setY(pos.getY()-1);
		wm.moveObj(this,new_pos);
		break;
	}
	if(pos.getX() == new_pos.getX() && pos.getY() == new_pos.getY()) {
		box.setCorner(new_pos);
	};
};

void MovingObstacle::out(){
	WorldManager &wm = WorldManager::getInstance();
	wm.markForDelete(this);
};

void MovingObstacle::spawnAmmo() {
	int num = rand()%100;
	if(num < 18) {
		int ammo = rand()%3 + 1;
		new AmmoDrop("second",ammo,pos);
	}
	else if(num < 36) {
		int ammo = rand()%3 + 1;
		new AmmoDrop("scatter",ammo,pos);
	}
	else if(num < 48) {
		int ammo = rand()%2 + 1;
		new AmmoDrop("rail",ammo,pos);
	}
	else if(num < 60) {
		int ammo = rand()%2 + 1;
		new AmmoDrop("rocket",ammo,pos);
	}
	else if(num < 65) {
		new LifeDrop(pos);
	}
};
