//
// RailGun.cpp
//

#include "RailGun.h"
#include "EnemyFactory.h"
#include "BigEnemy.h"
#include "Engine/LogManager.h"

#define MAX_LENGTH 12

RailGun::RailGun() {
	LogManager &lm = LogManager::getInstance();
	pos.setX(0);
	pos.setY(0);
	length = 1;
	max_length = MAX_LENGTH;
	duration = 18;
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this,"EVENT_STEP");
	damage = 3;
	done = false;
};

RailGun::RailGun(Position init_pos, int new_direction) {
	LogManager &lm = LogManager::getInstance();
	setPos(init_pos);
	length = 1;
	if(new_direction == LEFT || new_direction == RIGHT) {
		max_length = MAX_LENGTH * 2;
	}
	else if(new_direction == UP || new_direction == DOWN) {
		max_length = MAX_LENGTH;
	}
	duration = 18;
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this,"EVENT_STEP");
	damage = 3;
	direction = new_direction;
};

RailGun::RailGun(Position init_pos, int init_length, int new_direction) {
	LogManager &lm = LogManager::getInstance();
	setPos(init_pos);
	setSolid(true);
	altitude = 0;
	length = init_length;
	if(new_direction == LEFT || new_direction == RIGHT) {
		max_length = MAX_LENGTH * 2;
	}
	else if(new_direction == UP || new_direction == DOWN) {
		max_length = MAX_LENGTH;
	}
	duration = 18;
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this,"EVENT_STEP");
	damage = 4;
	direction = new_direction;
};

RailGun::RailGun(Position init_pos, int init_length, int init_duration, int new_direction) {
	LogManager &lm = LogManager::getInstance();
	setPos(init_pos);
	setSolid(true);
	altitude = 0;
	length = init_length;
	if(new_direction == LEFT || new_direction == RIGHT) {
		max_length = MAX_LENGTH * 2;
	}
	else if(new_direction == UP || new_direction == DOWN) {
		max_length = MAX_LENGTH;
	}
	duration = init_duration;
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this,"EVENT_STEP");
	damage = 4;
	direction = new_direction;
};

RailGun::~RailGun() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this,"EVENT_STEP");
};

int RailGun::eventHandler(Event *e) {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	if(e->getType() == "EVENT_STEP") {
		if(length < max_length && !done) {
			lm.writeLog("making a new one");
			newShot();
		}
		duration--;
		lm.writeLog("duration is now %d", duration);
		if(duration == 0) {
			wm.markForDelete(this);
		}
	};
	if(e->getType() == "EVENT_COLLISION") {
		/*wm.markForDelete(this);
		new RailGun(pos, length, duration);*/
	}
};

void RailGun::draw() {
	LogManager &lm = LogManager::getInstance();
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	Position new_pos = graph_man.worldToScreen(pos);
	lm.writeLog("world pos is drawn at %d, %d", pos.getX(), pos.getY());
	lm.writeLog("screen pos is drawn at %d, %d", new_pos.getX(), new_pos.getY());
	if(direction == LEFT || direction == RIGHT) {
		graph_man.drawCh(new_pos, '-');
	}
	else if(direction == UP || direction == DOWN) {
		graph_man.drawCh(new_pos, '|');
	}
};

void RailGun::newShot() {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	GameObject * enemy = NULL;
	if(ef.isBoss()) {
		lm.writeLog("Enemy is true");
		enemy = ef.getBoss();
	}
	if(direction == LEFT) {
		lm.writeLog("dir is left, length is %d, max length is %d", length, max_length);
		Position new_pos(pos.getX()-1,pos.getY());
		Position screen_pos = graph_man.worldToScreen(new_pos);
		if(screen_pos.getX() >= 0) {
			if(enemy && wm.boxAndPoint(new_pos,enemy->getBox())) {
				Bullet *new_one = new RailGun(new_pos, max_length, direction);
				wm.moveObj(new_one, new_pos);
			}
			else {
				lm.writeLog("boss pos");
				Bullet *new_one = new RailGun(new_pos, length + 1, direction);
				wm.moveObj(new_one, new_pos);
			}
		};
		done = true;
	}
	else if(direction == RIGHT) {
		Position new_pos(pos.getX()+1,pos.getY());
		Position screen_pos = graph_man.worldToScreen(new_pos);
		if(screen_pos.getX() < wm.getView().getHorizontal()) {
			if(enemy && wm.boxAndPoint(new_pos,enemy->getBox())) {
				Bullet *new_one = new RailGun(new_pos, max_length, direction);
				wm.moveObj(new_one, new_pos);
			}
			else {
				lm.writeLog("boss pos");
				Bullet *new_one = new RailGun(new_pos, length + 1, direction);
				wm.moveObj(new_one, new_pos);
			}
		};
		done = true;
	}
	else if(direction == UP) {
		Position new_pos(pos.getX(),pos.getY() - 1);
		Position screen_pos = graph_man.worldToScreen(new_pos);
		if(screen_pos.getY() >= 0) {
			if(enemy && wm.boxAndPoint(new_pos,enemy->getBox())) {
				Bullet *new_one = new RailGun(new_pos, max_length, direction);
				wm.moveObj(new_one, new_pos);
			}
			else {
				lm.writeLog("boss pos");
				Bullet *new_one = new RailGun(new_pos, length + 1, direction);
				wm.moveObj(new_one, new_pos);
			}
		};
		done = true;
	}
	else if(direction == DOWN) {
		Position new_pos(pos.getX(),pos.getY() + 1);
		Position screen_pos = graph_man.worldToScreen(new_pos);
		if(screen_pos.getY() < wm.getView().getVertical()) {
			if(enemy && wm.boxAndPoint(new_pos,enemy->getBox())) {
				Bullet *new_one = new RailGun(new_pos, max_length, direction);
				wm.moveObj(new_one, new_pos);
			}
			else {
				lm.writeLog("boss pos");
				Bullet *new_one = new RailGun(new_pos, length + 1, direction);
				wm.moveObj(new_one, new_pos);
			}
		};
		done = true;
	}
};
