//
// RocketExplosion.cpp
//

#include "RocketExplosion.h"
#include "Engine/WorldManager.h"
#include "Engine/GraphicsManager.h"

RocketExplosion::RocketExplosion(Position init_pos) {
	setType("rocket_explosion");
	setPos(init_pos);
	count = 15;
	altitude = 0;
	setSolid(true);
	Position corner_pos(init_pos.getX() - 4, init_pos.getY() - 2);
	box.setCorner(corner_pos);
	box.setVertical(5);
	box.setHorizontal(9);
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
};

RocketExplosion::~RocketExplosion() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
};

int RocketExplosion::eventHandler(Event *e) {
	WorldManager &wm = WorldManager::getInstance();
	if(e->getType() == "EVENT_STEP") {
		count--;
		if(count >= 12) {
			wm.pointDetect(this, pos);
		}
		else if(count >= 9) {
			for(int i = -1; i <= 1; i++) {
				for(int j = -2; j <= 2; j++) {
					Position temp_pos(pos.getX() + j, pos.getY() + i);
					wm.pointDetect(this, temp_pos);
				}
			}
		}
		else if(count >= 6) {
			for(int i = -2; i <= 2; i++) {
				for(int j = -4; j <= 4; j++) {
					Position temp_pos(pos.getX() + j, pos.getY() + i);
					wm.pointDetect(this, temp_pos);
				}
			}
		}
		else if(count == 0) {
			wm.markForDelete(this);
		}
	};
};

void RocketExplosion::draw() {
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	Position new_pos = graph_man.worldToScreen(pos);
	if(count >= 12) {
		graph_man.drawCh(new_pos,'#');
	}
	else if(count >= 9) {
		for(int i = -1; i <= 1; i++) {
			for(int j = -2; j <= 2; j++) {
				Position temp_pos(new_pos.getX() + j, new_pos.getY() + i);
				graph_man.drawCh(temp_pos, '#');
			}
		}
	}
	else if(count >= 6) {
		for(int i = -2; i <= 2; i++) {
			for(int j = -4; j <= 4; j++) {
				Position temp_pos(new_pos.getX() + j, new_pos.getY() + i);
				graph_man.drawCh(temp_pos, '#');
			}
		}
	}
	else if(count >= 3) {
		for(int i = -2; i <= -1; i++) {
			for(int j = -4; j <= 4; j++) {
				Position temp_pos(new_pos.getX() + j, new_pos.getY() + i);
				graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
			}
		}
		for(int j = -4; j <= -2; j++) {
			Position temp_pos(new_pos.getX() + j, new_pos.getY());
			graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
		}
		for(int j = 2; j <= 4; j++) {
			Position temp_pos(new_pos.getX() + j, new_pos.getY());
			graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
		}
		for(int i = 1; i <= 2; i++) {
			for(int j = -4; j <= 4; j++) {
				Position temp_pos(new_pos.getX() + j, new_pos.getY() + i);
				graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
			}
		}
	}
	else if(count >= 0) {
		for(int j = -4; j <= 4; j++) {
			Position temp_pos(new_pos.getX() + j, new_pos.getY() - 2);
			graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
		}
		for(int i = -1; i <= 1; i++) {
			Position temp_pos(new_pos.getX() - 4, new_pos.getY() + i);
			graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
		}
		for(int i = -1; i <= 1; i++) {
			Position temp_pos(new_pos.getX() + 4, new_pos.getY() + i);
			graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
		}
		for(int j = -4; j <= 4; j++) {
			Position temp_pos(new_pos.getX() + j, new_pos.getY() + 2);
			graph_man.drawCh(temp_pos, '#', COLOR_MAGENTA);
		}
	}
};
