//
// SecondBoss.cpp
//

#include "SecondBoss.h"
#include "EnemyShot.h"
#include "Bullet.h"
#include "EnemyFactory.h"
#include "EventBoss.h"
#include <stdlib.h>
#include <time.h>

SecondBoss::SecondBoss(Position init_pos) {
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	pos = init_pos;
	altitude = 0;
	dir = 2;
	ResourceManager &rm = ResourceManager::getInstance();
	p_sprite = rm.getSprite("secondBoss");
	sprite_index = 0;
	sprite_slowdown = 20;
	sprite_slowdown_count = 0;
	box.setCorner(pos);
	box.setHorizontal(p_sprite->getWidth());
	box.setVertical(p_sprite->getHeight());
	shot_counter = 300;
	spin_counter = 30;
	shot_dir = 1;
	move_counter = 0;
	health = 20;
	type = "SecondBoss";
	setSolid(true);
	srand(time(0));
};

SecondBoss::~SecondBoss() {	
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
};

int SecondBoss::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	GameManager &gm = GameManager::getInstance();
	WorldManager &wm = WorldManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		if(coll_e->getObj()->getType() == "bullet") {
			Bullet *bullet = static_cast <Bullet *> (coll_e->getObj());
			health = health - bullet->getDamage();
			EventBoss *e_boss = new EventBoss(health);
			wm.onEvent(e_boss);
		}
		if(coll_e->getObj()->getType() == "SO") {
			wm.markForDelete(coll_e->getObj());
		}
		if(health <= 0){
			wm.markForDelete(this);
			ef.wipeScreen();
			ef.setLevel(3);
			ef.startLevel();
		}
	}
	if(e->getType() == "EVENT_STEP") {
		move_counter--;
		shot_counter--;
		spin_counter--;
		if(move_counter<=0){
			move();
			move_counter = 15;
		}
		if(shot_counter<=0){
			int x = pos.getX();
			int y = pos.getY();
			Position pos1(x+2,y-1);
			Position pos2(x+3,y-1);
			Position pos3(x+4,y-1);
			Position pos4(x+7,y);
			Position pos5(x+7,y+1);
			Position pos6(x+7,y+2);
			Position pos7(x+7,y+3);
			Position pos8(x+7,y+4);
			Position pos9(x+6,y+7);
			Position pos10(x+5,y+7);
			Position pos11(x+4,y+7);
			Position pos12(x+3,y+7);
			Position pos13(x+2,y+7);
			Position pos14(x-1,y+6);
			Position pos15(x-1,y+5);
			Position pos16(x-1,y+4);
			Position pos17(x-1,y+3);
			Position pos18(x-1,y+2);
			Position pos19(x,y-1);
			Position pos20(x+1,y-1);
			switch(shot_dir){
			case 1:
				new EnemyShot(2,5,pos1);
				break;
			case 2:
				new EnemyShot(2,5,pos2);
				break;
			case 3:
				new EnemyShot(2,5,pos3);
				break;
			case 4:
				new EnemyShot(3,5,pos4);
				break;
			case 5:
				new EnemyShot(3,5,pos5);
				break;
			case 6:
				new EnemyShot(4,5,pos6);
				break;
			case 7:
				new EnemyShot(4,5,pos7);
				break;
			case 8:
				new EnemyShot(4,5,pos8);
				break;
			case 9:
				new EnemyShot(5,5,pos9);
				break;
			case 10:
				new EnemyShot(5,5,pos10);
				break;
			case 11:
				new EnemyShot(6,5,pos11);
				break;
			case 12:
				new EnemyShot(6,5,pos12);
				break;
			case 13:
				new EnemyShot(6,5,pos13);
				break;
			case 14:
				new EnemyShot(7,5,pos14);
				break;
			case 15:
				new EnemyShot(7,5,pos15);
				break;
			case 16:
				new EnemyShot(8,5,pos16);
				break;
			case 17:
				new EnemyShot(8,5,pos17);
				break;
			case 18:
				new EnemyShot(8,5,pos18);
				break;
			case 19:
				new EnemyShot(1,5,pos19);
				break;
			case 20:
				new EnemyShot(1,5,pos20);
				break;
			}
			shot_dir++;
			if(shot_dir>=21){
				shot_dir = 1;
				shot_counter = 300;
			}
			spin_counter = 20;
		}
	}
	return 0;
};

void SecondBoss::move() {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	int p_x = pos.getX();
	int p_y = pos.getY();

	switch(dir){
	case 2:
		p_x+=2;
		p_y--;
		break;
	case 4:
		p_x+=2;
		p_y++;
		break;
	case 6:
		p_x-=2;
		p_y++;
		break;
	case 8:
		p_x-=2;
		p_y--;
		break;
	}

	if(p_x+1 > wm.getBoundary().getHorizontal() && dir == 2){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 8;
		p_x-=2;
		p_y--;
	}
	else if(p_x+1 > wm.getBoundary().getHorizontal() && dir == 4){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 6;
		p_x-=2;
		p_y++;
	}
	else if(p_x < 0 && dir == 8){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 2;
		p_x+=2;
		p_y--;
	}
	else if(p_x < 0 && dir == 6){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 4;
		p_x+=2;
		p_y++;
	}
	else if(p_y > wm.getBoundary().getVertical() && dir == 4){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 2;
		p_x+=2;
		p_y--;
	}
	else if(p_y > wm.getBoundary().getVertical() && dir == 6){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 8;
		p_x-=2;
		p_y--;
	}
	else if(p_y < 0 && dir == 2){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 4;
		p_x+=2;
		p_y++;
	}
	else if(p_y < 0 && dir == 8){
		p_x = pos.getX();
		p_y = pos.getY();
		dir = 6;
		p_x-=2;
		p_y++;
	}

	Position new_pos(p_x,p_y);
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == new_pos.getY()) {
		box.setCorner(new_pos);
	};
};
