//
// SecondEnemy.cpp
//

#include "SecondEnemy.h"
#include "EnemyShot.h"
#include "LifeDrop.h"
#include <stdlib.h>
#include <time.h>
#include "EnemyFactory.h"

SecondEnemy::SecondEnemy(Position init_pos) {
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	srand(time(0));
	pos = init_pos;
	altitude = 0;
	dir = 0;
	ResourceManager &rm = ResourceManager::getInstance();
	p_sprite = rm.getSprite("secondEnemy");
	sprite_index = 0;
	sprite_slowdown = 60;
	sprite_slowdown_count = 0;
	box.setCorner(pos);
	box.setHorizontal(p_sprite->getWidth());
	box.setVertical(p_sprite->getHeight());
	shot_counter = rand()%250+100;
	move_counter = 30;
	type = "SecondEnemy";
	setSolid(true);
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.addEnemy(this);
	health = 3;
	hurt_count = -1;
};

SecondEnemy::~SecondEnemy() {	
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.removeEnemy(this);
};

int SecondEnemy::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	WorldManager &wm = WorldManager::getInstance();
	ResourceManager &rm = ResourceManager::getInstance();
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		lm.writeLog("Enemy recieved a collision event");
		if(coll_e->getObj()->getType() == "bullet") {
			Bullet *bullet = static_cast<Bullet *>(coll_e->getObj());
			int damage = bullet->getDamage();
			health -= damage;
			hurt_count = 10;
			if(health <= 0) {
				WorldManager &wm = WorldManager::getInstance();
				wm.markForDelete(this);
				EnemyFactory &ef = EnemyFactory::getInstance();
				ef.spawnAmmo(pos);
			};
		}
		if(coll_e->getObj()->getType() == "rocket_explosion") {
			WorldManager &wm = WorldManager::getInstance();
			wm.markForDelete(this);
			EnemyFactory &ef = EnemyFactory::getInstance();
			ef.spawnAmmo(pos);
		}
		if(coll_e->getObj()->getType() == "SO") {
			health--;
			hurt_count = 10;
			if(health <= 0) {
				WorldManager &wm = WorldManager::getInstance();
				wm.markForDelete(this);
				EnemyFactory &ef = EnemyFactory::getInstance();
				ef.spawnAmmo(pos);
			};
		}
	}
	if(e->getType() == "EVENT_STEP") {
		if(hurt_count == 10) {
			p_sprite = rm.getSprite("secondEnemyhurt");
			hurt_count--;
		}
		else if(hurt_count > 0) {
			hurt_count--;
		}
		else if(hurt_count == 0) {
			p_sprite = rm.getSprite("secondEnemy");
			hurt_count--;
		}
		move_counter--;
		shot_counter--;
		if(5 == (rand()%10))
			dir = rand()%8+1;
		if(move_counter<=0){
			move();
			move_counter = 40;
		}
		if(shot_counter<=0){
			Position shot_pos = pos;
			shot_pos.setX(pos.getX()+2);
			shot_pos.setY(pos.getY()-1);
			new EnemyShot(1,5,shot_pos);
			shot_pos.setX(pos.getX()+5);
			shot_pos.setY(pos.getY()+1);
			new EnemyShot(3,5,shot_pos);
			shot_pos.setX(pos.getX()+2);
			shot_pos.setY(pos.getY()+3);
			new EnemyShot(5,5,shot_pos);
			shot_pos.setX(pos.getX()-1);
			shot_pos.setY(pos.getY()+1);
			new EnemyShot(7,5,shot_pos);
			shot_counter = rand()%250+100;
		}
	}
	return 0;
};

void SecondEnemy::move() {
	WorldManager &wm = WorldManager::getInstance();
	LogManager &lm = LogManager::getInstance();
	int p_x = pos.getX();
	int p_y = pos.getY();
	
	switch(dir){
	case 1:
		p_y--;
		break;
	case 2:
		p_x+=2;
		p_y--;
		break;
	case 3:
		p_x+=2;
		break;
	case 4:
		p_x+=2;
		p_y++;
		break;
	case 5:
		p_y++;
		break;
	case 6:
		p_x-=2;
		p_y++;
		break;
	case 7:
		p_x-=2;
		break;
	case 8:
		p_x-=2;
		p_y--;
		break;
	}

	if(p_x>0 && p_x<wm.getBoundary().getHorizontal() && p_y>0 && p_y<wm.getBoundary().getVertical()){
		Position new_pos(p_x,p_y);
		wm.moveObj(this, new_pos);
		if(pos.getX() == new_pos.getX() && pos.getY() == new_pos.getY()) {
			box.setCorner(new_pos);
		}
	}
};
