//
// SmartEnemy.cpp
//

#include "SmartEnemy.h"
#include "EnemyFactory.h"
#include "LifeDrop.h"
#include <stdlib.h>

SmartEnemy::SmartEnemy(Position init_pos) {
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, "EVENT_STEP");
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	pos = init_pos;
	altitude = 0;
	dir = 0;
	shoot_dir = 0;
	ResourceManager &rm = ResourceManager::getInstance();
	p_sprite = rm.getSprite("smartEnemy");
	sprite_index = 0;
	sprite_slowdown = 60;
	sprite_slowdown_count = 0;
	box.setCorner(pos);
	box.setHorizontal(p_sprite->getWidth());
	box.setVertical(p_sprite->getHeight());
	shot_counter = rand()%600+250;
	move_counter = 40;
	type = "SmartEnemy";
	setSolid(true);
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.addEnemy(this);
	health = 4;
	hurt_count = -1;
};

SmartEnemy::~SmartEnemy() {	
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, "EVENT_STEP");
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.removeEnemy(this);
};

int SmartEnemy::eventHandler(Event *e) {
	LogManager &lm = LogManager::getInstance();
	ResourceManager &rm = ResourceManager::getInstance();
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		lm.writeLog("Enemy recieved a collision event");
		if(coll_e->getObj()->getType() == "bullet") {
			Bullet *bullet = static_cast<Bullet *>(coll_e->getObj());
			int damage = bullet->getDamage();
			health -= damage;
			hurt_count = 10;
			if(health <= 0) {
				WorldManager &wm = WorldManager::getInstance();
				wm.markForDelete(this);
				EnemyFactory &ef = EnemyFactory::getInstance();
				ef.spawnAmmo(pos);
			};
		}
		if(coll_e->getObj()->getType() == "rocket_explosion") {
			WorldManager &wm = WorldManager::getInstance();
			wm.markForDelete(this);
			EnemyFactory &ef = EnemyFactory::getInstance();
			ef.spawnAmmo(pos);
		}
		if(coll_e->getObj()->getType() == "SO") {
			health--;
			hurt_count = 10;
			if(health <= 0) {
				WorldManager &wm = WorldManager::getInstance();
				wm.markForDelete(this);
				EnemyFactory &ef = EnemyFactory::getInstance();
				ef.spawnAmmo(pos);
			};
		}
	}
	if(e->getType() == "EVENT_STEP") {
		if(hurt_count == 10) {
			p_sprite = rm.getSprite("smartEnemyhurt");
			hurt_count--;
		}
		else if(hurt_count > 0) {
			hurt_count--;
		}
		else if(hurt_count == 0) {
			p_sprite = rm.getSprite("smartEnemy");
			hurt_count--;
		}
		move_counter--;
		shot_counter--;
		if(move_counter<=0){
			move();
			move_counter = 40;
		}
		if(shot_counter<=0){
			int x = pos.getX();
			int y = pos.getY();
				Position pos1(x+1,y-1);
				Position pos2(x+3,y-1);
				Position pos3(x+3,y+1);
				Position pos4(x+3,y+3);
				Position pos5(x+1,y+3);
				Position pos6(x-1,y+3);
				Position pos7(x-1,y+1);
				Position pos8(x-1,y-1);
			switch(shoot_dir){
			case 1:
				new EnemyShot(1,5,pos1);
				break;
			case 2:
				new EnemyShot(2,5,pos2);
				break;
			case 3:
				new EnemyShot(3,5,pos3);
				break;
			case 4:
				new EnemyShot(4,5,pos4);
				break;
			case 5:
				new EnemyShot(5,5,pos5);
				break;
			case 6:
				new EnemyShot(6,5,pos6);
				break;
			case 7:
				new EnemyShot(7,5,pos7);
				break;
			case 8:
				new EnemyShot(8,5,pos8);
				break;
			}
			shot_counter = rand()%600+250;
	}
	return 0;
	}
};

void SmartEnemy::move() {
	WorldManager &wm = WorldManager::getInstance();
	EnemyFactory &ef = EnemyFactory::getInstance();
	LogManager &lm = LogManager::getInstance();
	int p_x = pos.getX();
	int p_y = pos.getY();
    int hero_pos_x = ef.getHero().getPos().getX();
	int hero_pos_y = ef.getHero().getPos().getY();

    if(p_x == hero_pos_x && p_y > hero_pos_y){
		dir = 1;
		shoot_dir = 1;
	}
	if(p_x < hero_pos_x && p_y > hero_pos_y){
		dir = 2;
		shoot_dir = 2;
	}
	if(p_x < hero_pos_x && p_y == hero_pos_y){
		dir = 3;
		shoot_dir = 3;
	}
	if(p_x < hero_pos_x && p_y < hero_pos_y){
		dir = 4;
		shoot_dir = 4;
	}
	if(p_x == hero_pos_x && p_y < hero_pos_y){
		dir = 5;
		shoot_dir = 5;
	}
	if(p_x > hero_pos_x && p_y < hero_pos_y){
		dir = 6;
		shoot_dir = 6;
	}
	if(p_x > hero_pos_x && p_y == hero_pos_y){
		dir = 7;
		shoot_dir = 7;
	}
    if(p_x > hero_pos_x && p_y > hero_pos_y){
		dir = 8;
		shoot_dir = 8;
	}
	if(p_x - hero_pos_x < 10 && p_x - hero_pos_x > -10 && p_y - hero_pos_y < 10 && p_y - hero_pos_y > -10)
		dir = 0;
	
	switch(dir){
	case 1:
		p_y--;
		break;
	case 2:
		p_x+=2;
		p_y--;
		break;
	case 3:
		p_x+=2;
		break;
	case 4:
		p_x+=2;
		p_y++;
		break;
	case 5:
		p_y++;
		break;
	case 6:
		p_x-=2;
		p_y++;
		break;
	case 7:
		p_x-=2;
		break;
	case 8:
		p_x-=2;
		p_y--;
		break;
	}

	Position new_pos(p_x,p_y);
	wm.moveObj(this, new_pos);
	if(pos.getX() == new_pos.getX() && pos.getY() == new_pos.getY()) {
		box.setCorner(new_pos);
	};
};

void SmartEnemy::spawnAmmo() {
	int num = rand()%100;
	if(num < 18) {
		int ammo = rand()%3 + 1;
		new AmmoDrop("second",ammo,pos);
	}
	else if(num < 36) {
		int ammo = rand()%3 + 1;
		new AmmoDrop("scatter",ammo,pos);
	}
	else if(num < 48) {
		int ammo = rand()%2 + 1;
		new AmmoDrop("rail",ammo,pos);
	}
	else if(num < 60) {
		int ammo = rand()%2 + 1;
		new AmmoDrop("rocket",ammo,pos);
	}
	else if(num < 65) {
		new LifeDrop(pos);
	}
};
