//
//StaticObstacle.cpp
//

#include "StaticObstacle.h"
#include "Engine/WorldManager.h"
#include "Engine/ResourceManager.h"
#include "Engine/EventCollision.h"
#include "Engine/Box.h"
#include "Hero.h"
#include "EnemyFactory.h"
#include "LifeDrop.h"
#include <stdlib.h>


StaticObstacle::StaticObstacle(Position init_pos){
	ResourceManager &rm = ResourceManager::getInstance();
	setSprite(rm.getSprite("SO"));
	pos = init_pos;
	setSolid(true);
	type = "SO";
	sprite_slowdown = 1;
	Box *new_box = new Box(pos,1,1);
	setBox(*new_box);
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.addEnemy(this);
};

StaticObstacle::~StaticObstacle(){
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.removeEnemy(this);
};

int StaticObstacle::eventHandler(Event *e){
	WorldManager &wm = WorldManager::getInstance();
	if(e->getType() == "EVENT_COLLISION") {
		EventCollision *coll_e = static_cast <EventCollision *> (e);
		if(coll_e->getObj()->getType() == "bullet" || coll_e->getObj()->getType() == "rocket_explosion") {
			wm.markForDelete(this);
			EnemyFactory &ef = EnemyFactory::getInstance();
			ef.spawnAmmo(pos);
		}
		else if(coll_e->getObj()->getType() == "hero" || coll_e->getObj()->getType() == "FirstEnemy" || coll_e->getObj()->getType() == "SecondEnemy" || coll_e->getObj()->getType() == "SmartEnemy" || coll_e->getObj()->getType() == "BigEnemy" ||coll_e->getObj()->getType() == "SecondBoss") {
			wm.markForDelete(this);
			moveHero(coll_e->getObj(), pos);
		};
	};
};

void StaticObstacle::moveHero(GameObject *p_go, Position new_pos) {
	Position hero_pos = p_go->getPos();
	Position new_hero_pos;
	int x = hero_pos.getX();
	int y = hero_pos.getY();
	if(hero_pos.getX() > new_pos.getX()) {
		x = x -2;
	}
	if(hero_pos.getX() + 2 < new_pos.getX()) {
		x = x + 2;
	}
	if(hero_pos.getY() > new_pos.getY()) {
		y--;
	}
	if(hero_pos.getY() + 2 < new_pos.getY()) {
		y++;
	}
	new_hero_pos.setX(x);
	new_hero_pos.setY(y);
	WorldManager &wm = WorldManager::getInstance();
	wm.moveObj(p_go, new_hero_pos);
};
