//
// test.cpp
//

// Engine includes
#include "Engine/GameManager.h"
#include "Engine/LogManager.h"
#include "Engine/GameObject.h"
#include "Engine/GraphicsManager.h"
#include "Engine/ResourceManager.h"
#include "Engine/Box.h"
#include "Engine/Position.h"
#include "EnemyFactory.h"
#include "Title.h"
#include "Lives.h"
#include "Ammo.h"
#include "GunLabel.h"
#include <stdio.h>

void populateWorld();
void loadResources();

int main(int argc, char *argv[]) {
  LogManager &log_manager = LogManager::getInstance();
  // Start up Game Manager
  GameManager &game_manager = GameManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  GraphicsManager &graph_man = GraphicsManager::getInstance();

  if (game_manager.startUp(false, true))  {
    log_manager.writeLog("Error starting game manager");
    game_manager.shutDown();
  }
  
  Position boundary_pos(0,0);
  Box boundary(boundary_pos, graph_man.getHorizontal() * 2, graph_man.getVertical() * 2);
  world_manager.setBoundary(boundary);
  
  log_manager.writeLog("horizontal - %d, vertical - %d", world_manager.getBoundary().getHorizontal(),world_manager.getBoundary().getVertical());
  
  Position view_pos(graph_man.getHorizontal() / 2,graph_man.getVertical() / 2);
  Box view(view_pos, graph_man.getHorizontal(), graph_man.getVertical());
  world_manager.setView(view);
  
  loadResources();
  log_manager.writeLog("Resources Loaded");
  
  log_manager.writeLog("Engine started");
  populateWorld();
  log_manager.writeLog("All Objects Created");


  game_manager.run();
  
  // Shut everything down
  game_manager.shutDown();
};

void populateWorld(void){ 
	new Lives;
	new Ammo;
	new GunLabel;
	EnemyFactory &ef = EnemyFactory::getInstance();
	ef.setLevel(1);
	ef.startLevel();
};

void loadResources(void) {
	ResourceManager &rm = ResourceManager::getInstance();
	rm.loadSprite("\\|/-X-/|\\", 3, 3, 1, COLOR_BLUE, "hero");
	rm.loadSprite("O", 1, 1, 1, COLOR_GREEN, "SO");
	rm.loadSprite("O", 1, 1, 1, COLOR_GREEN, "MO");
	rm.loadSprite("/=\\\\=/", 2, 3, 1, COLOR_RED, "firstEnemy");
	rm.loadSprite("/=\\\\=/", 2, 3, 1, COLOR_YELLOW, "firstEnemyhurt");
	rm.loadSprite(" ___ / X \\\\_X_/", 3, 5, 1, COLOR_RED, "secondEnemy");
	rm.loadSprite(" ___ / X \\\\_X_/", 3, 5, 1, COLOR_YELLOW, "secondEnemyhurt");
	rm.loadSprite("$S$S$SS$S$S$",2,3,2,COLOR_RED, "smartEnemy");
	rm.loadSprite("$S$S$SS$S$S$",2,3,2,COLOR_YELLOW, "smartEnemyhurt");
	rm.loadSprite("/)______(\\|        |<        ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (//)______(\\|        |<        ><  %%%%  ><  ****  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><  %%%%  ><        >| ______ |\\)      (/", 13, 10, 12, COLOR_RED, "bigEnemy");
	rm.loadSprite(" ______  _____  _____  _____  _____  _______  ______  _____  _____   _____ |  __  ||  ___||  ___||_   _||_   _||  __   ||  __  ||_   _||  __ \\ |  ___|| |__| || |___ | |      | |    | |  | |__|  || |  | |  | |  | |  | || |___ |  __  ||___  || |      | |    | |  |  __  _|| |  | |  | |  | |  | ||___  || |  | | ___| || |___  _| |_  _| |_ | |  \\ \\ | |__| | _| |_ | |__| | ___| ||_|  |_||_____||_____||_____||_____||_|   \\_||______||_____||_____/ |_____|", 6, 75, 1, COLOR_WHITE, "title");
	rm.loadSprite(" __      ______  ___        ___  ______  __          ______ |  |    |   ___| \\  \\      /  / |   ___||  |        |___   ||  |    |  |___   \\  \\    /  /  |  |___ |  |         ___|  ||  |    |   ___|   \\  \\  /  /   |   ___||  |        |   ___||  |___ |  |___     \\  \\/  /    |  |___ |  |___     |  |___ |______||______|     \\____/     |______||______|    |______|", 6, 60, 1, COLOR_WHITE, "level2");
	rm.loadSprite(" __      ______  ___        ___  ______  __          ______ |  |    |   ___| \\  \\      /  / |   ___||  |        |___   ||  |    |  |___   \\  \\    /  /  |  |___ |  |         ___|  ||  |    |   ___|   \\  \\  /  /   |   ___||  |        |___   ||  |___ |  |___     \\  \\/  /    |  |___ |  |___      ___|  ||______||______|     \\____/     |______||______|    |______|", 6, 60, 1, COLOR_WHITE, "level3");
	rm.loadSprite("WASD to move, Arrows to shoot, 1 or 2 to change weapons, ESC to quit", 1, 68, 1, COLOR_WHITE, "instructions");
	rm.loadSprite("Press 3 or 4 to change to new weapons, scatter gun and rail gun", 1, 63, 1, COLOR_WHITE, "instructions2");
	rm.loadSprite("Rockets are now available. Press 5 to use them", 1, 46, 1, COLOR_WHITE, "instructions3");
	rm.loadSprite(" |XXX- |     -\\ O@O \\\\ OOO \\\\ OOO \\-     | -///|  |///X |     X\\ OO@ \\\\ OOO \\\\ OOO \\-     | -///|  |///- |     -\\ OOO X\\ OO@ X\\ OOO X-     | -///|  |///- |     -\\ OOO \\\\ OOO \\\\ OO@ \\-     X -///X  |///- |     -\\ OOO \\\\ OOO \\\\ O@O \\-     | -XXX|  |///- |     -\\ OOO \\\\ OOO \\\\ @OO \\X     | X///|  |///- |     -X OOO \\X @OO \\X OOO \\-     | -///|  X///- X     -\\ @OO \\\\ OOO \\\\ OOO \\-     | -///| ", 7, 7, 8, COLOR_RED, "secondBoss");
};
